@@ -664,6 +664,20 @@ void Level::removeScrambleItem(int nScrambleItem)
   vec.erase(vec.begin() + nScrambleItem);
 }
 
+const std::string &Level::getNoteText(int nNote, Language lang) const
+{ 
+  const std::string &note = m_noteText[nNote][lang];
+  if (note == "" && lang != Language_International)
+    return m_noteText[nNote][Language_International];
+  return note;
+}
+
+void Level::setNoteText(int nNote, Language lang, const std::string &noteText)
+{ 
+  m_noteText[nNote][lang].assign(noteText); 
+}
+
+
 const Laser &Level::getLaser(int nLaser) const
 {
   return m_lasers[nLaser];
@@ -918,43 +932,6 @@ namespace
     return true;
   }
 
-  bool parseNumberList(const string &str,
-                       vector<int> *pNumberList)
-  {
-    pNumberList->clear();
-
-    int cur = 0;
-    int sz = str.size();
-    while (cur < sz) {
-      if (str[cur] != '(') {
-        return false;
-      }
-      cur++;
-
-      bool bEmpty = true;
-      int num = 0;
-      while (cur < sz && str[cur] >= '0' && str[cur] <= '9') {
-        bEmpty = false;
-        num *= 10;
-        num += (str[cur] - '0');
-        cur++;
-      }
-
-      if (cur < sz) {
-        if (str[cur] == ')') {
-          pNumberList->push_back(bEmpty ? -1 : num);
-          cur++;
-        } else {
-          return false;
-        }
-      } else {
-        return false;
-      }
-    }
-
-    return true;
-  }
-
   void unparseNumberList(const vector<int> &numberList,
                          string *pStr)
   {
@@ -971,6 +948,81 @@ namespace
   }
 }
 
+
+bool parseNumberList(const string &str,
+                     vector<int> *pNumberList)
+{
+    pNumberList->clear();
+
+    int cur = 0;
+    int sz = str.size();
+    while (cur < sz) {
+        if (str[cur] != '(') {
+            return false;
+        }
+        cur++;
+
+        bool bEmpty = true;
+        int num = 0;
+        while (cur < sz && str[cur] >= '0' && str[cur] <= '9') {
+            bEmpty = false;
+            num *= 10;
+            num += (str[cur] - '0');
+            cur++;
+        }
+
+        if (cur < sz) {
+            if (str[cur] == ')') {
+                pNumberList->push_back(bEmpty ? -1 : num);
+                cur++;
+            } else {
+                return false;
+            }
+        } else {
+            return false;
+        }
+    }
+
+    return true;
+}
+
+namespace
+{
+  class CommandParser {
+    string            m_cmd;
+    string::iterator  m_iter;
+
+  public:
+    CommandParser (const string &cmd)
+      : m_cmd(cmd), m_iter(m_cmd.begin())
+    {}
+
+    char get_char () {
+      char c = 0;
+      if (m_iter != m_cmd.end()) {
+        c = *m_iter;
+        ++m_iter;
+      }
+      return c;
+    }
+
+
+    bool get_bool(bool default_) {
+      unsigned char c = get_char();
+      switch (c) {
+      case 0xf8: return false;
+      case 0xf9: return true;
+      default: return default_;
+      }
+    }
+
+    int get_int(int default_) {
+        return default_;
+    }
+  };
+}
+
+
 bool parseLevel(const ByteVec &in, Level *pLevel, string *pMsg)
 {
   string msg;
@@ -1844,6 +1896,7 @@ bool parseLevel(const ByteVec &in, Level *pLevel, string *pMsg)
           case 'g': lang = Language_German;  break;
           case 'e': lang = Language_English; break;
           case 'f': lang = Language_French;  break;
+          case 'i': lang = Language_International; break;
           }
           if (lang != Language_Invalid) {
             if (item[2] == '"') {
