@@ -21,9 +21,10 @@
 #include "gui/GameMenu.hh"
 #include "gui/OptionsMenu.hh"
 #include "gui/LevelInspector.hh"
+#include "gui/HelpMenu.hh"
 #include "client.hh"
 #include "display.hh"
-#include "ecl.hh"
+#include "ecl_video.hh"
 #include "main.hh"
 #include "nls.hh"
 #include "server.hh"
@@ -40,34 +41,51 @@ namespace enigma { namespace gui {
 /* -------------------- GameMenu -------------------- */
     
     GameMenu::GameMenu (int zoomxpos_, int zoomypos_)
-    : zoomed(0),
+    : zoomed(0), complete(0),
       zoomxpos(zoomxpos_),
       zoomypos(zoomypos_)
     {
-        resume  = new gui::StaticTextButton(N_("Resume Level"), this);
-        restart = new gui::StaticTextButton(N_("Restart Level"), this);
+        const VMInfo *vminfo = video_engine->GetInfo();
+
+        resume  = new gui::StaticTextButton(N_("Resume Level (ESC)"), this);
+        restart = new gui::StaticTextButton(N_("Restart Level (Shift-F3)"), this);
         options = new gui::StaticTextButton(N_("Options"), this);
         info    = new gui::StaticTextButton(N_("Level Info"), this);
+        scrshot = new gui::StaticTextButton(N_("Screenshot (F10)"), this);
         abort   = new gui::StaticTextButton(N_("Abort Level"), this);
-    
-        add(resume,     Rect(0,0,200,40));
-        add(restart,    Rect(0,45,200,40));
-        add(options,    Rect(0,90,200,40));
-        add(info,       Rect(0,135,200,40));
-        add(abort,      Rect(0,180,200,40));
+        bosskey = new gui::StaticTextButton(N_("Exit Enigma (Shift-ESC)"), this);
+
+        if (vminfo->width < 640) {
+            add(resume,     Rect(0,0,145,40));
+            add(restart,    Rect(0,45,145,40));
+            add(options,    Rect(0,90,145,40));
+            add(info,       Rect(150,0,145,40));
+            add(abort,      Rect(150,45,145,40));
+            add(bosskey,    Rect(150,90,145,40));
+        } else {
+            add(resume,     Rect(0,0,270,40));
+            add(restart,    Rect(0,45,270,40));
+            add(options,    Rect(0,90,270,40));
+            add(info,       Rect(0,135,270,40));
+            add(scrshot,    Rect(0,180,270,40));
+            add(abort,      Rect(0,270,270,40));
+            add(bosskey,    Rect(0,315,270,40));
+        }
         center();
     }
     
     GameMenu::~GameMenu() {
         delete zoomed;
+        delete complete;
     }
     
     void GameMenu::draw_background(ecl::GC &gc) 
     {
-        const video::VMInfo *vminfo = video::GetInfo();
-    
+        const VMInfo *vminfo = video_engine->GetInfo();
+
         if (!zoomed) {
             Rect game_area   = display::GetGameArea();
+            complete = Grab(video_engine->GetScreen()->get_surface(), game_area);
             int  part_width  = game_area.w/3;
             int  part_height = (part_width*vminfo->height)/vminfo->width;
     
@@ -107,38 +125,63 @@ namespace enigma { namespace gui {
             // Be sure to redraw everything, or actors may appear on top
             // of the stones (actors are drawn in one pass and only
             // clipped to the screen boundary).
-            display::RedrawAll(video::GetScreen());
-    
+            display::RedrawAll(video_engine->GetScreen());
+
             // get the selected part from screen
-    //         SDL_Surface *back = video::GetScreen()->get_surface();
             Rect     src_area(game_area.x+x, game_area.y+y, part_width, part_height);
-            Surface *src = Grab(video::GetScreen()->get_surface(), src_area);
-    
-            // zoom multiple times for softer image
-    //         const double stepsize = 0.3;
-    //         for (double zoom = 0.4; zoom < 0.9; zoom += stepsize) {
-    //             int      sx  = round_down<int>(zoom * vminfo->width);
-    //             int      sy  = round_down<int>(zoom * vminfo->height);
-    //             Surface *tmp = src->zoom(sx, sy);
-    
-    //             delete src;
-    //             src = tmp;
-    //         }
+            Surface *src = Grab(video_engine->GetScreen()->get_surface(), src_area);
+
             zoomed = src->zoom(vminfo->width+1, vminfo->height+1);
             delete src;
         }
     
         ecl::blit(gc, 0,0, zoomed);
     }
-    
+
+    static const char *helptext_gamemenu[] = {
+        N_("F1:"), N_("Show this help"),
+        N_("F3:"), N_("Kill current marble"),
+        N_("Shift+F3:"), N_("Restart the current level"),
+        N_("Shift+Ctrl+F3:"), N_("Reload and restart the current level"),
+        N_("Right mouse button:"), N_("Return to level"),
+        " ", " ",
+        "FREE TEXT FOLLOWS",
+        N_("For a list of shortcut keys during the game, press F1 during the game."),
+        0
+    };
+
     bool GameMenu::on_event (const SDL_Event &e) 
     {
-        if (e.type == SDL_MOUSEBUTTONDOWN
-            && e.button.button == SDL_BUTTON_RIGHT)
-        {
+        if (e.type == SDL_MOUSEBUTTONDOWN && e.button.button == SDL_BUTTON_RIGHT) {
             Menu::quit();
             return true;
         }
+        if (e.type == SDL_KEYDOWN) {
+            Uint16 keymod = e.key.keysym.mod;
+            SDL_Keycode keysym = e.key.keysym.sym;
+            switch (keysym) {
+            case SDLK_F3: {
+                lev::Index *ind = lev::Index::getCurrentIndex();
+                if (keymod & KMOD_SHIFT & KMOD_CTRL) {
+                    lev::Proxy::releaseCache();
+                    client::Stop ();
+                    server::Msg_LoadLevel(ind->getCurrent(), false);
+                } else if (keymod & KMOD_SHIFT) {
+                    client::Stop ();
+                    server::Msg_LoadLevel(ind->getCurrent(), false);
+                } else {
+                    server::Msg_Command("suicide");
+                }
+                Menu::quit();
+                return true;
+                break; }
+            case SDLK_F1: {
+                displayHelp(helptext_gamemenu, 200);
+                draw_all();
+                break; }
+            default: break;
+            };
+        }
         return false;
     }
     
@@ -159,10 +202,9 @@ namespace enigma { namespace gui {
             client::Stop ();
             server::Msg_LoadLevel(ind->getCurrent(), false);
             Menu::quit();
-            
         }
         else if (w == options) {
-            enigma::gui::ShowOptionsMenu (0);
+            enigma::gui::ShowOptionsMenu (0, true);
             invalidate_all();
 //            Menu::quit();
         }
@@ -172,5 +214,12 @@ namespace enigma { namespace gui {
             invalidate_all();
 //            Menu::quit();
         }
+        else if (w == bosskey) {
+            client::Msg_Command("abort");
+            app.bossKeyPressed = true;
+        }
+        else if (w == scrshot) {
+            video_engine->Screenshot(server::LoadedProxy->getNextScreenshotPath(), complete);
+        }
     }
 }} // namespace enigma::gui
