@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006 Ronald Lamprecht
+ * Copyright (C) 2006,2007,2008,2009 Ronald Lamprecht
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,9 +19,11 @@
 #include "errors.hh"
 #include "gui/LevelInspector.hh"
 #include "main.hh"
-#include "enigma.hh"
 #include "nls.hh"
+#include "resource_cache.hh"
 #include "video.hh"
+#include "ecl_font.hh"
+#include "ecl_system.hh"
 #include "ecl_util.hh"
 #include "gui/MonospacedLabel.hh"
 #include "gui/LevelPreviewCache.hh"
@@ -30,7 +32,6 @@
 #include "lev/ScoreManager.hh"
 #include "StateManager.hh"
 
-
 #include <vector>
 
 using namespace ecl;
@@ -46,7 +47,7 @@ namespace enigma { namespace gui {
             theRatingMgr->setIntelligence(theLevel, value);
         }
     
-        string get_text(int value) const  {
+        std::string get_text(int value) const  {
             return strf("%d", value);
         }
     public:
@@ -68,7 +69,7 @@ namespace enigma { namespace gui {
             theRatingMgr->setDexterity(theLevel, value);
         }
     
-        string get_text(int value) const  {
+        std::string get_text(int value) const  {
             return strf("%d", value);
         }
     public:
@@ -90,7 +91,7 @@ namespace enigma { namespace gui {
             theRatingMgr->setPatience(theLevel, value);
         }
     
-        string get_text(int value) const  {
+        std::string get_text(int value) const  {
             return strf("%d", value);
         }
     public:
@@ -112,7 +113,7 @@ namespace enigma { namespace gui {
             theRatingMgr->setKnowledge(theLevel, value);
         }
     
-        string get_text(int value) const  {
+        std::string get_text(int value) const  {
             return strf("%d", value);
         }
     public:
@@ -134,7 +135,7 @@ namespace enigma { namespace gui {
             theRatingMgr->setSpeed(theLevel, value);
         }
     
-        string get_text(int value) const  {
+        std::string get_text(int value) const  {
             return strf("%d", value);
         }
     public:
@@ -154,9 +155,12 @@ namespace enigma { namespace gui {
          }
          void set_value(int value) { 
              theScoreMgr->setRating(theLevel, value);
+             if (get_parent() != NULL) {
+                get_parent()->invalidate_all();
+             }
          }
      
-         string get_text(int value) const  {
+         std::string get_text(int value) const  {
              return (value == -1) ? "-" : ecl::strf("%d", value);
          }
      public:
@@ -170,36 +174,41 @@ namespace enigma { namespace gui {
          lev::Proxy *theLevel;        
      };
     
-LevelInspector::LevelInspector(lev::Proxy *aLevel):
-        levelProxy(aLevel), annotation (new TextField()),
+LevelInspector::LevelInspector(lev::Proxy *aLevel, bool showDeveloperInfo):
         back (new StaticTextButton(N_("Ok"), this)),
-        screenshot (new StaticTextButton(N_("Screenshot"), this))
+        screenshot (new StaticTextButton(N_("Screenshot"), this)),
+        annotation (new TextField()), levelProxy(aLevel),        
+        isDeveloperMode(showDeveloperInfo)
     {
         bool didGenerate;  // dummy
-        previewImage = LevelPreviewCache::instance()->getPreview(aLevel, true, didGenerate);
-        const video::VMInfo *vminfo = video::GetInfo();
-        vspacing = vminfo->height < 500 ? 2 :(vminfo->height < 650 ? 3 : 4);
-        vspacing2 = vminfo->height < 500 ? 16 :(vminfo->height < 650 ? 14 : 16);
-        vmargin = vminfo->height < 500 ? 10 :(vminfo->height < 650 ?  20 : 30);
-        hmargin = vminfo->width < 660 ? 10 : (vminfo->width < 900 ? 20 : 30);
-        bool highres = vminfo->height > 650 ? true : false;
+        const VMInfo *vminfo = video_engine->GetInfo();
+        previewImage =
+            LevelPreviewCache::instance()->getPreview(aLevel, vminfo->thumb, true, didGenerate);
+        const int vshrink = vminfo->width < 640 ? 1 : 0;
+        vspacing = vminfo->height < 400 ? 1 :(vminfo->height < 500 ? 2 :(vminfo->height < 650 ? 3 : 4));
+        vspacing2 = vminfo->height < 400 ? 8 :(vminfo->height < 500 ? 16 :(vminfo->height < 650 ? 14 : 16));
+        vmargin = vminfo->height < 400 ? 5 :(vminfo->height < 500 ? 10 :(vminfo->height < 650 ?  20 : 30));
+        hmargin = vminfo->width < 640 ? 5 : (vminfo->width < 660 ? 10 : (vminfo->width < 900 ? 20 : 30));
         bool lowres = vminfo->height < 600 ? true : false;
+        bool lowlowres = vminfo->height < 400 ? true : false;
     
-        add(back, Rect(vminfo->width-130-2*hmargin,vminfo->height-50,130,35));
-        add(screenshot, Rect(vminfo->width-260-3*hmargin,vminfo->height-50,130,35));
+        add(back, Rect(vminfo->width-(vshrink?65:130)-2*hmargin,
+                       vminfo->height-(vshrink?25:50),vshrink?65:130,vshrink?17:35));
+        add(screenshot, Rect(vminfo->width-(vshrink?130:260)-3*hmargin,
+                             vminfo->height-(vshrink?25:50),vshrink?65:130,vshrink?17:35));
     
         try {
             aLevel->loadMetadata(true);
         }
         catch (XLevelLoading &err) {
-            std::vector<string> lines;
+            std::vector<std::string> lines;
             std::string errmsg = _("Server Error: could not load level '")
-                                   + aLevel->getNormLevelPath() + "'\n"
+                                   + aLevel->getNormFilePath() + "'\n"
                                    + err.what();
             ecl::split_copy (errmsg, '\n', back_inserter(lines));
-            int x     = 60;
-            int y     = 60;
-            int yskip = 25;
+            int x     = vshrink?30:60;
+            int y     = vshrink?30:60;
+            int yskip = vshrink?12:25;
             for (unsigned i=0; i<lines.size(); ++i) {
                 add(new Label(lines[i], HALIGN_LEFT), Rect(x, y, vminfo->width-80,yskip));
                 y += yskip;
@@ -210,24 +219,29 @@ LevelInspector::LevelInspector(lev::Proxy *aLevel):
         std::string tmp, tmp2;
         lev::RatingManager *theRatingMgr = lev::RatingManager::instance();
         lev::ScoreManager  *theScoreMgr = lev::ScoreManager::instance();
-        withEasy = aLevel->hasEasymode();
+        withEasy = aLevel->hasEasyMode();
+        ratingInherited = theScoreMgr->isRatingInherited(aLevel);
         ecl::Font *menufont = enigma::GetFont("menufont");
         levelPathString = 
                 (levelProxy->getNormPathType() == lev::Proxy::pt_resource) ?
-                levelProxy->getAbsLevelPath() : levelProxy->getNormLevelPath();
+                levelProxy->getAbsLevelPath() : levelProxy->getNormFilePath();
         // substitute all backslashes by slashes
         for (std::string::size_type slpos = levelPathString.find('\\');
                 slpos != std::string::npos; slpos = levelPathString.find('\\')) 
             levelPathString.replace(slpos, 1, 1, '/');
-        BuildVList authorT(this, Rect(hmargin,vmargin,110,25), vspacing);
+        BuildVList authorT(this, Rect(hmargin,vmargin,vshrink?55:110,vshrink?12:25), vspacing);
         authorT.add(new Label(N_("Level: "), HALIGN_RIGHT));
         authorT.add(new Label((""), HALIGN_RIGHT)); // subtitle
         authorT.add(new Label(N_("Author: "), HALIGN_RIGHT));
         authorT.add(new Label(N_("Contact: "), HALIGN_RIGHT));
         authorT.add(new Label(N_("Homepage: "), HALIGN_RIGHT));
-        
-        BuildVList author(this, Rect(hmargin+110+10,vmargin,
-                vminfo->width-(hmargin+110+10)-10-(vminfo->thumbw+10)-hmargin,25), vspacing);
+
+        BuildVList author(this, Rect(hmargin + (vshrink ? 55 + 5 : 110 + 10), vmargin,
+                                     vminfo->width - (hmargin + (vshrink ? 55 + 5 : 110 + 10)) -
+                                         (vshrink ? 5 : 10) -
+                                         (vminfo->thumb.width + (vshrink ? 5 : 10)) - hmargin,
+                                     vshrink ? 12 : 25),
+                          vspacing);
         tmp = levelProxy->getTitle();
         tmp2 = levelProxy->getLocalizedString("title");
         if (tmp != tmp2)
@@ -239,14 +253,14 @@ LevelInspector::LevelInspector(lev::Proxy *aLevel):
         author.add(new Label(tmp, HALIGN_LEFT)); // subtitle
         author.add(new Label(levelProxy->getAuthor(), HALIGN_LEFT));
         
-        BuildVList address(this, Rect(hmargin+110+10,vmargin+3*(25+vspacing),
-                vminfo->width-(hmargin+110+10)-hmargin,25), vspacing);
+        BuildVList address(this, Rect(hmargin+(vshrink?55+5:110+10),vmargin+3*((vshrink?12:25)+vspacing),
+                vminfo->width-(hmargin+(vshrink?55+5:110+10))-hmargin,vshrink?12:25), vspacing);
         address.add(new Label(levelProxy->getContact(), HALIGN_LEFT));
         address.add(new Label(levelProxy->getHomepage(), HALIGN_LEFT));
         
-        BuildVList ratingPubT(this, Rect(hmargin+65,vmargin+5*25+4*vspacing+vspacing2, 130,25), 2);
+        BuildVList ratingPubT(this, Rect(hmargin+(vshrink?32:65),vmargin+5*(vshrink?12:25)+4*vspacing+vspacing2,vshrink?65:130,vshrink?12:25), vshrink?1:2);
         ratingPubT.add(new Label(N_("Public Ratings"), HALIGN_CENTER));
-        BuildVList ratingPubST(this, Rect(hmargin,vmargin+6*25+5*vspacing+vspacing2, 130,25), 2);
+        BuildVList ratingPubST(this, Rect(hmargin,vmargin+6*(vshrink?12:25)+5*vspacing+vspacing2,vshrink?65:130,vshrink?12:25), vshrink?1:2);
         ratingPubST.add(new Label(N_("Intelligence: "), HALIGN_RIGHT));
         ratingPubST.add(new Label(N_("Dexterity: "), HALIGN_RIGHT));
         ratingPubST.add(new Label(N_("Patience: "), HALIGN_RIGHT));
@@ -255,7 +269,7 @@ LevelInspector::LevelInspector(lev::Proxy *aLevel):
         if (!lowres) ratingPubST.add(new Label(N_("Difficulty: "), HALIGN_RIGHT));
 
         
-        BuildVList ratingPub(this, Rect(hmargin+130+15,vmargin+6*25+5*vspacing+vspacing2, 30,25), 2);
+        BuildVList ratingPub(this, Rect(hmargin+(vshrink?65+7:130+15),vmargin+6*(vshrink?12:25)+5*vspacing+vspacing2,vshrink?15:30,vshrink?12:25),vshrink?1:2);
         if (WizardMode) {
             ratingPub.add(new IntelligenceButton(aLevel));
             ratingPub.add(new DexterityButton(aLevel));
@@ -272,10 +286,10 @@ LevelInspector::LevelInspector(lev::Proxy *aLevel):
             if (!lowres) ratingPub.add(new MonospacedLabel(ratingToString(theRatingMgr->getDifficulty(aLevel)).c_str(),'8', " 0123456789", HALIGN_CENTER));
         }
         
-        BuildVList scoresT(this, Rect(vminfo->width/2-100-20,vmargin+5*25+4*vspacing+vspacing2,100,25), 2);
+        BuildVList scoresT(this, Rect(vminfo->width/2-(vshrink?50+10:100+20),vmargin+5*(vshrink?12:25)+4*vspacing+vspacing2,vshrink?50:100,vshrink?12:25), vshrink?1:2);
         scoresT.add(new Label(N_("Scores"), HALIGN_RIGHT));
 
-        BuildVList scoresST(this, Rect(vminfo->width/2-100-20,vmargin+6*25+5*vspacing+vspacing2,100,25), 2);
+        BuildVList scoresST(this, Rect(vminfo->width/2-(vshrink?50+10:100+20),vmargin+6*(vshrink?12:25)+5*vspacing+vspacing2,vshrink?50:100,vshrink?12:25), vshrink?1:2);
         scoresST.add(new Label(N_("You: "), HALIGN_RIGHT));
         scoresST.add(new Label(N_("World: "), HALIGN_RIGHT));
 // TRANSLATORS: PAR = professional average rate - an expression used by golfers
@@ -284,8 +298,8 @@ LevelInspector::LevelInspector(lev::Proxy *aLevel):
         scoresST.add(new Label(N_("Solved %: "), HALIGN_RIGHT));
         if (!lowres) scoresST.add(new Label(N_("Solved #: "), HALIGN_RIGHT));
     
-        BuildVList scores(this, Rect(vminfo->width/2-15+(withEasy?0:20),
-                vmargin+6*25+5*vspacing+vspacing2,(withEasy?117:54),25), 2);
+        BuildVList scores(this, Rect(vminfo->width/2-(vshrink?7-(withEasy?0:10):15-(withEasy?0:20)),
+                vmargin+6*(vshrink?12:25)+5*vspacing+vspacing2,vshrink?(withEasy?53:27):(withEasy?117:54),vshrink?12:25), vshrink?1:2);
         scores.add(new MonospacedLabel(scoreToString(theScoreMgr->getBestUserScore(aLevel, DIFFICULTY_EASY),
             theScoreMgr->getBestUserScore(aLevel, DIFFICULTY_HARD),aLevel,true).c_str(),'8', " 0123456789", HALIGN_CENTER));
         scores.add(new MonospacedLabel(scoreToString(theRatingMgr->getBestScoreEasy(aLevel),
@@ -303,9 +317,9 @@ LevelInspector::LevelInspector(lev::Proxy *aLevel):
             (ecl::strf("%5d", theRatingMgr->getNumSolvedDifficult(aLevel)) + " ").c_str(),
             '8', " 0123456789", HALIGN_CENTER));        
         
-        BuildVList versionT(this, Rect(vminfo->width-100/2-90-2*hmargin,vmargin+5*25+4*vspacing+vspacing2,100,25), 2);
+        BuildVList versionT(this, Rect(vminfo->width-(vshrink?50/2+45:100/2+90)-2*hmargin,vmargin+5*(vshrink?12:25)+4*vspacing+vspacing2,vshrink?50:100,vshrink?12:25), vshrink?1:2);
         versionT.add(new Label(N_("Version"), HALIGN_CENTER));
-        BuildVList versionST(this, Rect(vminfo->width-110-90-2*hmargin,vmargin+6*25+5*vspacing+vspacing2,110,25), 2);
+        BuildVList versionST(this, Rect(vminfo->width-(vshrink?55+45:110+90)-2*hmargin,vmargin+6*(vshrink?12:25)+5*vspacing+vspacing2,vshrink?55:110,vshrink?12:25), vshrink?1:2);
         if (!lowres || aLevel->getLevelStatus() == lev::STATUS_RELEASED)
             versionST.add(new Label(N_("Score: "), HALIGN_RIGHT));
         else
@@ -317,7 +331,7 @@ LevelInspector::LevelInspector(lev::Proxy *aLevel):
         versionST.add(new Label(N_("Control: "), HALIGN_RIGHT));
         versionST.add(new Label(N_("Target: "), HALIGN_RIGHT));
 
-        BuildVList version(this, Rect(vminfo->width-80-2*hmargin,vmargin+6*25+5*vspacing+vspacing2,80+2*hmargin,25), 2);
+        BuildVList version(this, Rect(vminfo->width-(vshrink?40:80)-2*hmargin,vmargin+6*(vshrink?12:25)+5*vspacing+vspacing2,(vshrink?40:80)+2*hmargin,vshrink?12:25), vshrink?1:2);
         if (!lowres || aLevel->getLevelStatus() == lev::STATUS_RELEASED)
             version.add(new MonospacedLabel(ecl::strf("%6d", aLevel->getScoreVersion()).c_str(),
                     '8', " 0123456789", HALIGN_LEFT));
@@ -334,7 +348,7 @@ LevelInspector::LevelInspector(lev::Proxy *aLevel):
                 '8', " 0123456789", HALIGN_LEFT));
         version.add(new MonospacedLabel(ecl::strf("%6d", aLevel->getRevisionNumber()).c_str(),
                 '8', " 0123456789", HALIGN_LEFT));
-        if (!lowres)
+        if (!lowres) {
             if (aLevel->getLevelStatus() == lev::STATUS_RELEASED)
                 version.add(new Label(N_("released"), HALIGN_LEFT));
             else if (aLevel->getLevelStatus() == lev::STATUS_STABLE)
@@ -345,6 +359,7 @@ LevelInspector::LevelInspector(lev::Proxy *aLevel):
                 version.add(new Label(N_("experimental"), HALIGN_LEFT));
             else
                 version.add(new Label(N_("unknown"), HALIGN_LEFT));
+        }
         
         switch (aLevel->getControl()) {
             case lev::force:
@@ -374,13 +389,14 @@ LevelInspector::LevelInspector(lev::Proxy *aLevel):
         int levelPathLines = 0;
         int annotationLines = 0; 
         int compatibilityLines = 0; 
-        int vnext = vmargin+ (lowres?11:12)*25+(lowres?9:10)*vspacing+2*vspacing2;
-        int textwidth = vminfo->width-3*hmargin-110-10;
+        int idLines = 0; 
+        int vnext = vmargin+(lowres?11:12)*(vshrink?12:25)+(lowlowres?5:lowres?9:10)*vspacing+2*vspacing2;
+        int textwidth = vminfo->width-3*hmargin-(vshrink?55+5:110+10);
         dispatchBottomLines(bestScoreHolderLines, creditsLines, dedicationLines,
-                levelPathLines, annotationLines, compatibilityLines,
-                (vminfo->height-vnext-vmargin-25-vspacing2)/27, textwidth);
+                levelPathLines, annotationLines, compatibilityLines, idLines,
+                (vminfo->height-vnext-vmargin-(vshrink?12:25)-vspacing2)/(vshrink?13:27), textwidth);
         if (bestScoreHolderLines == 1) {
-            add(new Label(N_("World Record Holders: "), HALIGN_RIGHT),Rect(hmargin,vnext,200,25));
+            add(new Label(N_("World Record Holders: "), HALIGN_RIGHT),Rect(hmargin,vnext,vshrink?100:200,vshrink?12:25));
             std::string holders;
             if (withEasy) {
                 holders = theRatingMgr->getBestScoreEasyHolder(aLevel);
@@ -392,76 +408,164 @@ LevelInspector::LevelInspector(lev::Proxy *aLevel):
                 holders += " -";
             else
                 holders += theRatingMgr->getBestScoreDifficultHolder(aLevel);
-            add(new Label(holders, HALIGN_LEFT), Rect(hmargin+200+10,vnext,textwidth-90,25));
-            vnext += 25 + vspacing;
+            Label *wrLabel = new Label(holders, HALIGN_LEFT);
+            add(wrLabel, Rect(hmargin+(vshrink?100+5:200+10),vnext,textwidth-(vshrink?45:90),vshrink?12:25));
+            if (!wrLabel->text_fits()) {
+                int cutEasy = 0;
+                int cutDiff = 0;
+                std::string diffHolders = theRatingMgr->getBestScoreDifficultHolder(aLevel);
+                if (withEasy) {
+                    std::string easyHolders = theRatingMgr->getBestScoreEasyHolder(aLevel);
+                    bool hasEasyHolders = !easyHolders.empty();
+                    bool hasDiffHolders = !diffHolders.empty();
+                    int limit = 10;
+                    do {
+                        std::string cutHolders;
+                        wrLabel->set_text(easyHolders);
+                        if (!wrLabel->text_fits(0.48)) {
+                            cutHolders = theRatingMgr->getBestScoreEasyHolder(aLevel, ++cutEasy);
+                            if (cutHolders.empty())
+                                cutEasy--;
+                            else
+                                easyHolders = cutHolders;
+                        }
+                        wrLabel->set_text(diffHolders);
+                        if (!wrLabel->text_fits(0.48)) {
+                            cutHolders = theRatingMgr->getBestScoreDifficultHolder(aLevel, ++cutDiff);
+                            if (cutHolders.empty())
+                                cutDiff--;
+                            else
+                                diffHolders = cutHolders;
+                        }
+                        holders = (hasEasyHolders ? easyHolders : std::string("  - "))
+                            + " / " + (hasDiffHolders ? diffHolders : std::string(" -"));
+                        wrLabel->set_text(holders);
+                        limit--;
+                    } while (!wrLabel->text_fits() && limit > 0);
+                } else {
+                    std::string cutHolders;
+                    do {
+                        cutHolders = theRatingMgr->getBestScoreDifficultHolder(aLevel, ++cutDiff);
+                        wrLabel->set_text(cutHolders);
+                    } while (!wrLabel->text_fits());
+                    if (cutHolders.empty()) {
+                        // we did cut off to many holders, take last attempt even if it was too long
+                        wrLabel->set_text(theRatingMgr->getBestScoreDifficultHolder(aLevel, --cutDiff));
+                    }
+                }
+            }
+            vnext += (vshrink?12:25) + vspacing;
         }
         if (creditsLines >= 1) {
-            add(new Label(N_("Credits: "), HALIGN_RIGHT),Rect(hmargin,vnext,110,25));
+            add(new Label(N_("Credits: "), HALIGN_RIGHT),Rect(hmargin,vnext,vshrink?55:110,vshrink?12:25));
             std::string creditsString = levelProxy->getCredits(true);
             for (int i = 0; i< creditsLines; i++) {
                 std::string::size_type breakPos = breakString(menufont, creditsString, 
                         " ", textwidth);
-                add(new Label(creditsString.substr(0,breakPos), HALIGN_LEFT), Rect(hmargin+110+10,vnext,textwidth,25));
+                add(new Label(creditsString.substr(0,breakPos), HALIGN_LEFT), Rect(hmargin+(vshrink?55+5:110+10),vnext,textwidth,vshrink?12:25));
                 creditsString = creditsString.substr(breakPos);
-                vnext += (25 + vspacing);
+                vnext += ((vshrink?12:25) + vspacing);
             }
         }
         if (dedicationLines >= 1) {
-            add(new Label(N_("Dedication: "), HALIGN_RIGHT),Rect(hmargin,vnext,110,25));
+            add(new Label(N_("Dedication: "), HALIGN_RIGHT),Rect(hmargin,vnext,vshrink?55:110,vshrink?12:25));
             std::string dedicationString = levelProxy->getDedication(true);
             for (int i = 0; i< dedicationLines; i++) {
                 std::string::size_type breakPos = breakString( menufont, dedicationString, 
                         " ", textwidth);
-                add(new Label(dedicationString.substr(0,breakPos), HALIGN_LEFT), Rect(hmargin+110+10,vnext,textwidth,25));
+                add(new Label(dedicationString.substr(0,breakPos), HALIGN_LEFT), Rect(hmargin+(vshrink?55+5:110+10),vnext,textwidth,vshrink?12:25));
                 dedicationString = dedicationString.substr(breakPos);
-                vnext += (25 + vspacing);
+                vnext += ((vshrink?12:25) + vspacing);
             }
         }
         if (levelPathLines >= 1) {
-            add(new Label(N_("Level Path: "), HALIGN_RIGHT),Rect(hmargin,vnext,110,25));
-            std::string workString = levelPathString;
+            add(new Label(N_("Level Path: "), HALIGN_RIGHT),Rect(hmargin,vnext,vshrink?55:110,vshrink?12:25));
+            bool localSeparator = (levelProxy->getNormPathType() == lev::Proxy::pt_resource) ||
+                        (levelProxy->getNormPathType() == lev::Proxy::pt_absolute);
+            std::string workString = localSeparator ? ecl::BeautifyPath(levelPathString) : levelPathString;
             for (int i = 0; i< levelPathLines - 1; i++) {
                 std::string::size_type breakPos = breakString(menufont, workString, 
-                        "/", textwidth);
-                add(new Label(workString.substr(0,breakPos), HALIGN_LEFT), Rect(hmargin+110+10,vnext,textwidth,25));
+                        localSeparator ? ecl::PathSeparator : "/", textwidth);
+                add(new Label(workString.substr(0,breakPos), HALIGN_LEFT), Rect(hmargin+(vshrink?55+5:110+10),vnext,textwidth,vshrink?12:25));
                 workString = workString.substr(breakPos);
-                vnext += (25 + vspacing);
+                vnext += ((vshrink?12:25) + vspacing);
             }
             // show as much as possible from last line
             if (menufont->get_width(workString.c_str()) > textwidth) {
                 // show the filename at the end - skip leading parts if necessary
-                add(new Label(workString, HALIGN_RIGHT), Rect(hmargin+110+10,vnext,textwidth,25));
+                add(new Label(workString, HALIGN_RIGHT), Rect(hmargin+(vshrink?55+5:110+10),vnext,textwidth,vshrink?12:25));
             } else {
                 // display up to the last character
-                add(new Label(workString, HALIGN_LEFT), Rect(hmargin+110+10,vnext,textwidth,25));
+                add(new Label(workString, HALIGN_LEFT), Rect(hmargin+(vshrink?55+5:110+10),vnext,textwidth,vshrink?12:25));
             }
-            vnext += (25 + vspacing);
+            vnext += ((vshrink?12:25) + vspacing);
+        }
+        if (idLines >= 1) {
+            add(new Label(N_("Id: "), HALIGN_RIGHT),Rect(hmargin,vnext,110,25));
+            add(new Label(levelProxy->getId(), HALIGN_LEFT),Rect(hmargin+110+10, vnext, textwidth, 25));
+            add(new Label(N_("Load time: "), HALIGN_LEFT),Rect(hmargin+110+10+200+10, vnext, 110, 25));
+            add(new Label(ecl::strf("%g", levelProxy->getLoadtime()), HALIGN_LEFT),Rect(hmargin+110+10+200+10+90+10, vnext, 110, 25));
+            vnext += (25 + vspacing)*idLines;
         }
         if (compatibilityLines >= 1) {
-            add(new Label(N_("Compatibility: "), HALIGN_RIGHT),Rect(hmargin,vnext,110,25));
+            add(new Label(N_("Compatibility: "), HALIGN_RIGHT),Rect(hmargin,vnext,vshrink?55:110,vshrink?12:25));
             std::string compString = ecl::strf("Enigma v%.2f  /  ", levelProxy->getEnigmaCompatibility()) +
                     GetGameTypeName(levelProxy->getEngineCompatibility());
-            add(new Label(compString , HALIGN_LEFT),Rect(hmargin+110+10, vnext, textwidth, 25));
-            vnext += (25 + vspacing)*compatibilityLines;
+            add(new Label(compString , HALIGN_LEFT),Rect(hmargin+(vshrink?55+5:110+10), vnext, textwidth, vshrink?12:25));
+            vnext += ((vshrink?12:25) + vspacing)*compatibilityLines;
         }
+        annotation->set_text(app.state->getAnnotation(levelProxy->getId())); // field needs to initialized for saves
         if (annotationLines >= 1) {
-            add(new Label(N_("Annotation: "), HALIGN_RIGHT),Rect(hmargin,vnext,110,25));
-            annotation->set_text(app.state->getAnnotation(levelProxy->getId()));
-            add(annotation, Rect(hmargin+110+10, vnext, textwidth, 25));
-            vnext += (25 + vspacing)*annotationLines;
+            add(new Label(N_("Annotation: "), HALIGN_RIGHT),Rect(hmargin,vnext,vshrink?55:110,vshrink?12:25));
+            add(annotation, Rect(hmargin+(vshrink?55+5:110+10), vnext, textwidth, vshrink?12:25));
+            vnext += ((vshrink?12:25) + vspacing)*annotationLines;
         }
         vnext += vspacing2 - vspacing;
-        add(new Label(N_("Rating: "), HALIGN_RIGHT),Rect(hmargin,vnext,110,25));
-        add(new RatingButton(aLevel),Rect(hmargin+110+10,vnext,40,25));
-        add(new Label(N_("Average: "), HALIGN_RIGHT),Rect(hmargin+110+10+40+20,vnext,105,25));
-        add(new Label(theRatingMgr->getAverageRating(aLevel), HALIGN_RIGHT),Rect(hmargin+110+10+40+18+105+6,vnext,31,25));
+        add(new Label(N_("Rating: "), HALIGN_RIGHT),Rect(hmargin,vnext,vshrink?55:110,vshrink?12:25));
+        add(new RatingButton(aLevel),Rect(hmargin+(vshrink?55+5:110+10),vnext,vshrink?20:40,vshrink?12:25));
+        add(new Label(N_("Average: "), HALIGN_RIGHT),Rect(hmargin+(vshrink?55+5+20+10:110+10+40+20),vnext,vshrink?52:105,vshrink?12:25));
+        add(new Label(theRatingMgr->getAverageRating(aLevel), HALIGN_RIGHT),Rect(hmargin+(vshrink?55+5+20+9+52+3:110+10+40+18+105+6),vnext,vshrink?15:31,vshrink?12:25));
     }
     
     LevelInspector::~LevelInspector () {
     }
     
+    bool LevelInspector::isEndDeveloperMode() {
+        return isDeveloperMode;
+    }
+    
     bool LevelInspector::on_event (const SDL_Event &e) {
-        return false;
+        bool handled = false;
+        if (e.type == SDL_KEYDOWN) {
+            handled=true;
+            switch (e.key.keysym.sym) {
+            case SDLK_F2:
+                if (!isDeveloperMode) {
+                    if (!annotation->getText().empty() || 
+                            !app.state->getAnnotation(levelProxy->getId()).empty()) {
+                        app.state->setAnnotation(levelProxy->getId(), annotation->getText());
+                    }
+                    LevelInspector m(levelProxy, true);
+                    m.manage();
+                    if (m.isEndDeveloperMode()) {
+                        // reinit user input fields
+                        annotation->set_text(app.state->getAnnotation(levelProxy->getId()));
+                        invalidate_all();
+                    } else {
+                        Menu::quit();
+                    }
+                } else {
+                    if (!annotation->getText().empty() || 
+                            !app.state->getAnnotation(levelProxy->getId()).empty()) {
+                        app.state->setAnnotation(levelProxy->getId(), annotation->getText());
+                    }
+                    Menu::quit();
+                }
+                break;
+            default: handled=false; break;
+            }
+        }
+        return handled;
     }
     
     void LevelInspector::on_action(gui::Widget *w) {
@@ -471,6 +575,7 @@ LevelInspector::LevelInspector(lev::Proxy *aLevel):
                     !app.state->getAnnotation(levelProxy->getId()).empty()) {
                 app.state->setAnnotation(levelProxy->getId(), annotation->getText());
             }
+            isDeveloperMode = false;
             Menu::quit();
         } else if (w == screenshot) {
             ScreenshotViewer m(levelProxy);
@@ -480,21 +585,30 @@ LevelInspector::LevelInspector(lev::Proxy *aLevel):
     }
     
     void LevelInspector::draw_background(ecl::GC &gc) {
-        const video::VMInfo *vminfo = video::GetInfo();
-        video::SetCaption (("Enigma - Level Inspector"));
-        blit(gc, 0,0, enigma::GetImage("menu_bg", ".jpg"));
-        blit(gc, vminfo->width-vminfo->thumbw-10-hmargin, vmargin, previewImage);
+        const VMInfo *vminfo = video_engine->GetInfo();
+        const int vshrink = vminfo->width < 640 ? 1 : 0;
+        set_caption((std::string("Enigma - Level ") + (isDeveloperMode ? "Developer " : "") +
+                     "Inspector").c_str());
+        blit(gc, vminfo->mbg_offsetx, vminfo->mbg_offsety, enigma::GetImage("menu_bg", ".jpg"));
+        blit(gc, vminfo->width - vminfo->thumb.width - 10 - hmargin, vmargin, previewImage);
         Surface *img_hard = enigma::GetImage("completed");
         if (withEasy) {
             Surface *img_easy = enigma::GetImage("completed-easy");
-            blit (gc, vminfo->width/2-4, vmargin+5*25+4*vspacing+vspacing2, img_easy);
-            blit (gc, vminfo->width/2-4+63, vmargin+5*25+4*vspacing+vspacing2, img_hard);
+            blit (gc, vminfo->width/2-(vshrink?2:4), vmargin+5*(vshrink?12:25)+4*vspacing+vspacing2, img_easy);
+            blit (gc, vminfo->width/2-(vshrink?2-31:4-63), vmargin+5*(vshrink?12:25)+4*vspacing+vspacing2, img_hard);
         } else {
-            blit (gc, vminfo->width/2-4+20, vmargin+5*25+4*vspacing+vspacing2, img_hard);
+            blit (gc, vminfo->width/2-(vshrink?2-10:4-20), vmargin+5*(vshrink?12:25)+4*vspacing+vspacing2, img_hard);
+        }
+        Surface *img_outdated = enigma::GetImage("ic-outdated-120x78");
+        ratingInherited = lev::ScoreManager::instance()->isRatingInherited(levelProxy);
+        if (ratingInherited) {
+            int numLines = vminfo->height < 400 ? 10 :(vminfo->height < 500 ? 14 :(vminfo->height < 650 ? 18 : 19));
+            blit (gc, hmargin+(vshrink?55+5+20:110+10+40), vmargin + numLines*(vshrink?18:25) +
+                    (numLines-3)*vspacing + 3*vspacing2 - 6, img_outdated);
         }
     }
     
-    void LevelInspector::tick(double dtime) {
+    void LevelInspector::tick(double /*dtime*/) {
     }
 
     std::string LevelInspector::ratingToString(int value) {
@@ -531,9 +645,15 @@ LevelInspector::LevelInspector(lev::Proxy *aLevel):
     
     std::string LevelInspector::scoreToString(int score, lev::Proxy *aLevel) {
         if (aLevel->getScoreUnit() == lev::duration) 
-            if (score >= 0 && score <= (99*60+59))
-                return ecl::strf("%2d:%02d", score/60, score%60);
-            else
+            if (score >= 0) {
+                int hours = score / 3600;
+                int minutes = (score - 3600 * hours) / 60;
+                int seconds = score % 60;
+                if (hours > 0)
+                    return ecl::strf("%2d:%02d'", hours, minutes);
+                else
+                    return ecl::strf("%2d'%02d\"", minutes, seconds);
+            } else
                 return "  -  ";
         else
             if (score >= 0 && score <= 9999)
@@ -544,12 +664,16 @@ LevelInspector::LevelInspector(lev::Proxy *aLevel):
     
     void LevelInspector::dispatchBottomLines(int &bestScoreHolderLines, 
             int &creditsLines, int &dedicationLines, int &levelPathLines,
-            int &annotationLines, int &compatibilityLines, int numLines, int width) {
-        enum botType {holder, credits, dedication, path, annotation, compatibility};
+            int &annotationLines, int &compatibilityLines, int &idLines, int numLines, int width) {
+        enum botType {holder, credits, dedication, path, annotation, compatibility, id};
         const int sequenceSize = 13;
-        botType sequence[sequenceSize] = {credits, dedication, annotation, path,
+        botType sequence1[sequenceSize] = {credits, dedication, annotation, path,
                 holder, annotation, path, compatibility, credits, dedication, 
                 annotation, credits, annotation};
+        botType sequence2[sequenceSize] = {id, path, compatibility, holder, path, 
+                annotation, annotation, credits, dedication,
+                credits, dedication, annotation, annotation};
+        botType *sequence = isDeveloperMode ? sequence2 : sequence1;
         int j = 0;
         std::string creditsString = levelProxy->getCredits(true);
         std::string dedicationString = levelProxy->getDedication(true);
@@ -595,6 +719,10 @@ LevelInspector::LevelInspector(lev::Proxy *aLevel):
                         compatibilityLines++;
                         assigned = true;
                         break;
+                    case id: 
+                        idLines++;
+                        assigned = true;
+                        break;
                 }
                 
             } while (!assigned && j < sequenceSize);
