@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2002,2003,2004,2005,2006 Daniel Heck
+ * Copyright (C) 2006,2007,2008,2009,2010 Ronald Lamprecht
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -21,16 +22,17 @@
 
 #include "gui/HelpMenu.hh"
 #include "gui/LevelPackMenu.hh"
+#include "gui/LevelPackComposer.hh"
 
-#include "ecl.hh"
 #include "game.hh"
 #include "main.hh"
 #include "nls.hh"
 #include "options.hh"
+#include "resource_cache.hh"
 #include "server.hh"
-#include "sound.hh"
+#include "MusicManager.hh"
 #include "StateManager.hh"
-#include "video.hh"
+#include "lev/PersistentIndex.hh"
 #include "lev/Index.hh"
 
 using namespace std;
@@ -40,22 +42,6 @@ using namespace ecl;
 namespace enigma { namespace gui {
 /* -------------------- Level Menu -------------------- */
     
-    struct LevelMenuConfig {
-        int buttonw, ibuttonw, buttonh;
-        int lbuttonw, lbuttonh;
-        ecl::Rect previewarea;
-        int thumbsy;                // y coordinate of thumbnail window
-        int leftborder;
-    
-        LevelMenuConfig (const ecl::Rect &screen)
-        : buttonw (140), ibuttonw (90), buttonh (35),
-          lbuttonw (140), lbuttonh (100),
-          previewarea (10, 60, screen.w-50, screen.h-130),
-          thumbsy (60),
-          leftborder (10)
-        {}
-    };
-    
     LevelMenu::LevelMenu()
     : but_advancemode (new AdvanceModeButton),
       but_next       (new ImageButton("ic-next", "ic-next1", this)),
@@ -68,42 +54,81 @@ namespace enigma { namespace gui {
       lbl_levelinfo  (new Label("", HALIGN_LEFT)),
       shown_text_ttl(-1.0), main_quit (false)
     {
+        static struct SpacingConfig {
+            int hmargin_info, vmargin, vgap_info, vsize_info;
+            int hmargin_prev, hgap_prev_nav, vgap_info_prev, vgap_prev_but;
+            int hsize_nav, hmargin_nav, vsize_nav, vgap_info_nav, vgap_nav_nav, vgap_nav1_nav2;
+            int hsize_button_small, hsize_button_large, hgap_button, vsize_button, vmargin_bottom;
+        } param[] = {
+            {  // VTS_16 (320x240)
+                2, 5, 10, 14,
+                5, 5, 5, 1,
+                10, 5, 25, 10, 5, 45,
+                45, 70, 5, 17, 7
+            },
+            {  // VTS_32 (640x480)
+                5, 10, 20, 28,
+                10, 10, 10, 2,
+                20, 10, 50, 20, 10, 90,
+                90, 140, 10, 35, 15
+            },
+            {  // VTS_40 (800x600)
+                5, 10, 20, 28,
+                10, 10, 10, 0,
+                20, 10, 50, 20, 10, 120,
+                90, 140, 10, 35, 15
+            },
+            {  // VTS_48 (960x720)  VM_1024x768
+                5, 10, 20, 28,
+                10, 10, 10, 0,
+                20, 10, 50, 20, 10, 120,
+                90, 140, 10, 35, 15
+            },
+            {  // VTS_64 (1280x960)
+                15, 10, 25, 28,
+                15, 25, 15, 0,
+                20, 15, 50, 40, 10, 120,
+                90, 140, 10, 35, 15
+            }
+        };
+
+        const VMInfo &vminfo = *video_engine->GetInfo();
+        const int vshrink = vminfo.width < 640 ? 1 : 0;
+        VideoTileType vtt = vminfo.tt;
+
+        int preview_y = param[vtt].vmargin + 2*param[vtt].vgap_info + param[vtt].vgap_info_prev;
+        ecl::Rect previewarea(param[vtt].hmargin_prev, preview_y,
+                vminfo.width - param[vtt].hmargin_nav - param[vtt].hsize_nav - param[vtt].hgap_prev_nav, 
+                vminfo.height - preview_y - param[vtt].vgap_prev_but - param[vtt].vsize_button - param[vtt].vmargin_bottom);
+        
         HList *hl, *hll, *hlr ;
     
-        const video::VMInfo &vminfo = *video::GetInfo();
-    
-    
-        // Levelmenu configuration
-        const int Y2 = 10;          // y position for information area
-        const int Y3 = vminfo.height-50; // y position for bottom button row
-        LevelMenuConfig c (Rect (0, 0, vminfo.width, vminfo.height));
-    
         but_difficulty->set_listener (this);
     
-    
         // Create buttons
         hll = new HList;
-        hll->set_spacing (10);
+        hll->set_spacing (param[vtt].hgap_button);
         hll->set_alignment (HALIGN_CENTER, VALIGN_TOP);
-        hll->set_default_size (c.ibuttonw, c.buttonh);
+        hll->set_default_size (param[vtt].hsize_button_small, param[vtt].vsize_button);
         hll->add_back (but_advancemode);
         hll->add_back (but_next);
         hll->add_back (but_difficulty);
         
         hlr = new HList;
-        hlr->set_spacing (10);
+        hlr->set_spacing (param[vtt].hgap_button);
         hlr->set_alignment (HALIGN_CENTER, VALIGN_TOP);
-        hlr->set_default_size (c.buttonw, c.buttonh);
+        hlr->set_default_size (param[vtt].hsize_button_large, param[vtt].vsize_button);
         hlr->add_back (but_levelpack);
         hlr->add_back (but_back);
         
         hl = new HList;
-        hl->set_spacing (10);
+        hl->set_spacing (vshrink?5:10);
         hl->set_alignment (HALIGN_CENTER, VALIGN_TOP);
-        hl->set_default_size (2*c.buttonw + 10, c.buttonh);
+        hl->set_default_size (2*param[vtt].hsize_button_large + param[vtt].hgap_button, param[vtt].vsize_button);
         hl->add_back (hll);
         hl->add_back (hlr);
-        this->add (hl, Rect(c.leftborder, Y3, vminfo.width-20, c.buttonh));
+        this->add (hl, Rect(0, vminfo.height - param[vtt].vsize_button - param[vtt].vmargin_bottom,
+                vminfo.width, param[vtt].vsize_button));
             
         // Add navigation buttons
         pgup     = new ImageButton("ic-up", "ic-up1", this);
@@ -111,32 +136,35 @@ namespace enigma { namespace gui {
         start    = new ImageButton("ic-top", "ic-top1", this);
         end      = new ImageButton("ic-bottom", "ic-bottom1", this);
     
-        Rect r(vminfo.width-30, c.thumbsy, 20, 50);
-        r.y = c.thumbsy;
+        Rect r(vminfo.width - param[vtt].hmargin_nav - param[vtt].hsize_nav,
+                param[vtt].vmargin + 2*param[vtt].vgap_info + param[vtt].vgap_info_nav,
+                param[vtt].hsize_nav, param[vtt].vsize_nav);
         add (pgup, r);
-        r.y += 60;
+        r.y += param[vtt].vsize_nav + param[vtt].vgap_nav_nav;
         add (pgdown, r);
-        r.y = c.thumbsy + 240;
+        r.y += param[vtt].vsize_nav + param[vtt].vgap_nav1_nav2;
         add (start, r);
-        r.y += 60;
+        r.y += param[vtt].vsize_nav + param[vtt].vgap_nav_nav;
         add (end, r);
     
         // Information area
         hl = new HList;
         hl->add_back (lbl_levelname, List::EXPAND);
         hl->add_back (lbl_lpinfo, List::TIGHT);
-        this->add (hl, Rect (5, Y2, vminfo.width - 10, 28));
+        this->add (hl, Rect (param[vtt].hmargin_info, param[vtt].vmargin, 
+                vminfo.width - 2*param[vtt].hmargin_info, param[vtt].vsize_info));
     
         hl_info_stat = new HList;
         hl_info_stat->add_back (lbl_levelinfo, List::EXPAND); //Rect (c.leftborder, Y2+20,305, 28));
         hl_info_stat->add_back (lbl_statistics, List::TIGHT);
-        this->add (hl_info_stat, Rect (5, Y2+20, vminfo.width - 10, 28));
+        this->add (hl_info_stat, Rect (param[vtt].hmargin_info, param[vtt].vmargin + param[vtt].vgap_info, 
+                vminfo.width - 2*param[vtt].hmargin_info, param[vtt].vsize_info));
     
         // Prepare level selection widget
         levelwidget = new LevelWidget();
         levelwidget->set_listener(this);
-        levelwidget->realize (c.previewarea);
-        levelwidget->set_area (c.previewarea);
+        levelwidget->realize (previewarea);
+        levelwidget->set_area (previewarea);
     
         this->add (levelwidget);
         
@@ -166,10 +194,12 @@ namespace enigma { namespace gui {
         N_("Escape:"),              N_("Skip to main menu"),
         "F1:",                      N_("Show this help"),
         "F5:",                      0, // see below
+        "F6:",                      N_("Add all to clipboard"),
+        "F7:",                      N_("Update levelpack"),
         N_("Arrows:"),              N_("Select level"),
         N_("Return:"),              N_("Play selected level"),
         N_("Back/Space:"),          N_("Previous/next levelpack"),
-        "u",                        N_("Mark current level as Unsolved"),
+        "u",                        N_("Mark current level as unsolved"),
     //    "s",                        N_("Mark current level as Solved"),
         N_("Alt+Return:"),          N_("Switch between fullscreen and window"),
         N_("Left click:"),               N_("Play selected level"),
@@ -200,6 +230,22 @@ namespace enigma { namespace gui {
                 case SDLK_F5:
                     next_unsolved();
                     break;
+                case SDLK_F6:
+                    /*if (lev::Index::getCurrentIndex() != NULL) {
+                        lev::Variation var;
+                        for (int pos = 0; pos < lev::Index::getCurrentIndex()->size(); pos++) {
+                            var = lev::Index::getCurrentIndex()->getVariation(pos);
+                            clipboard->appendProxy(lev::Index::getCurrentIndex()->getProxy(pos),
+                                    var.ctrl, var.unit, var.target, var.extensions);
+                        }
+                    }*/
+                    gui::LevelPackComposer::addAllFromIndexToClipboard(lev::Index::getCurrentIndex());
+                    break;
+                case SDLK_F7:
+                    lev::Index::getCurrentIndex()->updateFromFolder();
+                    lev::Index::getCurrentIndex()->updateFromProxies();
+                    draw_all();
+                    break;
                 case SDLK_u: {
                     lev::ScoreManager::instance()->markUnsolved(lev::Index::getCurrentProxy(), 
                             app.state->getInt("Difficulty"));
@@ -227,12 +273,10 @@ namespace enigma { namespace gui {
             int ilevel = ind->getCurrentPosition();
             if (w->lastModifierKeys() & KMOD_CTRL && w->lastModifierKeys() & KMOD_SHIFT) {
                 // force a reload from file
-                lev::Proxy * curProxy = lev::Proxy::loadedLevel();
-                if (curProxy != NULL)
-                    curProxy->release();
+                lev::Proxy::releaseCache();
             }
     
-            if ((unsigned)ilevel < ind->size()) {
+            if (ilevel < (int)ind->size()) {
                 if (ind->mayPlayLevel(ilevel+1)) {
                     game::StartGame();
                     ilevel = ind->getCurrentPosition();
@@ -255,7 +299,8 @@ namespace enigma { namespace gui {
         } else if (w == end) {
             levelwidget->end();
         } else if (w == but_next) {
-            next_unsolved();
+            next_unsolved(); 
+            // if we jump to first level the view is not updated to levelwidget->start()
         } else if (w == but_levelpack) {
             main_quit = false;
             Menu::quit();
@@ -276,8 +321,8 @@ namespace enigma { namespace gui {
         lev::ScoreManager *scm = lev::ScoreManager::instance();
         lev::Proxy *curProxy = ind->getCurrent();
         int difficulty = app.state->getInt("Difficulty");
-        
-        lbl_lpinfo->set_text(ecl::strf(_("%s: %d levels"),
+
+        lbl_lpinfo->set_text(ecl::strf(nls::ntranslate("%s: %d level", "%s: %d levels", size).c_str(),
                 ind->getName().c_str(), size));
     
         if (size == 0) {
@@ -287,8 +332,6 @@ namespace enigma { namespace gui {
             lbl_levelinfo->set_text ("-");
         }
         else {
-            int iselected = ind->getCurrentPosition();
-    
             // Display levelpack statistics (percentage of solved levels)
     
             if (app.state->getInt("NextLevelMode") == lev::NEXT_LEVEL_NOT_BEST) {
@@ -312,7 +355,7 @@ namespace enigma { namespace gui {
                 // use the inspector to check the absolute path!
                 lbl_levelname->set_text(ecl::strf("#%d: %s (%s)",
                           ind->getCurrentLevel(), curProxy->getTitle().c_str(), 
-                          curProxy->getNormLevelPath().c_str()));
+                          curProxy->getNormFilePath().c_str()));
             } else {
                 lbl_levelname->set_text(ecl::strf("#%d: %s",
                           ind->getCurrentLevel(), curProxy->getTitle().c_str()));
@@ -324,7 +367,7 @@ namespace enigma { namespace gui {
             }
             else {
                 // TODO prepare for scores that are not time based!
-                char txt[200];
+                // char txt[200];
                 lev::RatingManager *ratingMgr = lev::RatingManager::instance();
                 int wr_time    = ratingMgr->getBestScore(curProxy, difficulty);
                 int par_time   = ratingMgr->getParScore(curProxy, difficulty);
@@ -333,20 +376,18 @@ namespace enigma { namespace gui {
                 string wr_name = ratingMgr->getBestScoreHolder(curProxy, difficulty);
                 bool  wr_name_displayed = false;
     
-                string your_time;
-                string wr_text;
+                std::string your_time;
+                std::string wr_text;
     
                 if (best_user_time>0) {
-                    your_time = strf(_("Your time: %d:%02d"),
-                             best_user_time/60, best_user_time%60);
+                    your_time = _("Your time: ") + ecl::timeformat(best_user_time);
     
                     if (wr_time>0) {
                         int below = wr_time - best_user_time;
                         if (below == 0)
                             wr_text = _("That's world record.");
                         else if (below>0)
-                            wr_text = strf(_("That's %d:%02d below world record."),
-                                    below/60, below%60);
+                            wr_text = ecl::strf(_("That's %s below world record."), ecl::timeformat(below).c_str());
                     }
                 }
     
@@ -355,10 +396,10 @@ namespace enigma { namespace gui {
                         wr_name_displayed = true;
                     } else
                         if (is_par || par_time < 0)
-                            wr_text = strf(_("World record: %d:%02d"), wr_time/60, wr_time%60);
+                            wr_text = _("World record: ") + ecl::timeformat(wr_time);
                         else
-                            wr_text = strf(_("Par: %d:%02d World record: %d:%02d"), 
-                                    par_time/60, par_time%60, wr_time/60, wr_time%60);
+                            wr_text = _("Par: ") + ecl::timeformat(par_time) + " " +  
+                                    _("World record: ") + ecl::timeformat(wr_time);
                 }
     
                 if (!your_time.empty())
@@ -371,11 +412,12 @@ namespace enigma { namespace gui {
                         if (!tmp.empty())
                             wr_name = tmp;
                         if (is_par || par_time < 0)
-                            wr_text = strf(_("World record by %s: %d:%02d"), 
-                                    wr_name.c_str(), wr_time/60, wr_time%60);
+                            wr_text = ecl::strf(_("World record by %s: "), wr_name.c_str()) +
+                                    ecl::timeformat(wr_time);
                         else
-                            wr_text = strf(_("Par: %d:%02d World record by %s: %d:%02d"), 
-                                    par_time/60, par_time%60, wr_name.c_str(), wr_time/60, wr_time%60);
+                            wr_text = _("Par: ") + ecl::timeformat(par_time) + " " +
+                                    ecl::strf(_("World record by %s: "), wr_name.c_str()) + 
+                                    ecl::timeformat(wr_time);
                     }
                     lbl_levelinfo->set_text(your_time + wr_text);
                 } while (!hl_info_stat->fits() && wr_name_displayed && (wr_cut < 20));
@@ -392,10 +434,12 @@ namespace enigma { namespace gui {
     
     void LevelMenu::draw_background(ecl::GC &gc) 
     {
-        video::SetCaption(("Enigma - Level Menu"));
-        sound::PlayMusic (options::GetString("MenuMusicFile"));
+        const VMInfo *vminfo = video_engine->GetInfo();
+
+        set_caption(("Enigma - Level Menu"));
+        sound::StartMenuMusic();
     
-        blit(gc, 0,0, enigma::GetImage("menu_bg", ".jpg"));
+        blit(gc, vminfo->mbg_offsetx, vminfo->mbg_offsety, enigma::GetImage("menu_bg", ".jpg"));
     }
 
     void LevelMenu::next_unsolved() 
@@ -460,40 +504,44 @@ namespace enigma { namespace gui {
         
     void AdvanceModeButton::update() {
         switch (app.state->getInt("NextLevelMode")) {
-        case lev::NEXT_LEVEL_UNSOLVED :
-            ImageButton::set_images("ic-unsolved", "par");
-            break;
-        case lev::NEXT_LEVEL_OVER_PAR :
-            ImageButton::set_images("par", "ic-worldrecord");
-            break;
-        case lev::NEXT_LEVEL_NOT_BEST :
-            ImageButton::set_images("ic-worldrecord", "ic-strictlynext");
-            break;
-        case lev::NEXT_LEVEL_STRICTLY : // use as default, too
-        default:
-            ImageButton::set_images("ic-strictlynext","ic-unsolved");
-            break;        
+            case lev::NEXT_LEVEL_UNSOLVED :
+                ImageButton::set_images("ic-unsolved", "par");
+                break;
+            case lev::NEXT_LEVEL_OVER_PAR :
+                ImageButton::set_images("par", "ic-worldrecord");
+                break;
+            case lev::NEXT_LEVEL_NOT_BEST :
+                ImageButton::set_images("ic-worldrecord", "ic-strictlynext");
+                break;
+            case lev::NEXT_LEVEL_STRICTLY : // use as default, too
+            default:
+                ImageButton::set_images("ic-strictlynext","ic-unsolved");
+                break;        
         }
     }
     
-    void AdvanceModeButton::on_action(Widget *) 
-    {
+    void AdvanceModeButton::on_action(Widget *) {
         switch (app.state->getInt("NextLevelMode")) {
-        case lev::NEXT_LEVEL_STRICTLY :
-            app.state->setProperty("NextLevelMode", lev::NEXT_LEVEL_UNSOLVED);
-            break;        
-        case lev::NEXT_LEVEL_UNSOLVED :
-            app.state->setProperty("NextLevelMode", lev::NEXT_LEVEL_OVER_PAR);
-            break;
-        case lev::NEXT_LEVEL_OVER_PAR :
-            app.state->setProperty("NextLevelMode", lev::NEXT_LEVEL_NOT_BEST);
-            break;
-        case lev::NEXT_LEVEL_NOT_BEST :
-        default:
-            app.state->setProperty("NextLevelMode", lev::NEXT_LEVEL_STRICTLY);
+            case lev::NEXT_LEVEL_STRICTLY :
+                app.state->setProperty("NextLevelMode", lev::NEXT_LEVEL_UNSOLVED);
+                break;        
+            case lev::NEXT_LEVEL_UNSOLVED :
+                app.state->setProperty("NextLevelMode", lev::NEXT_LEVEL_OVER_PAR);
+                break;
+            case lev::NEXT_LEVEL_OVER_PAR :
+                app.state->setProperty("NextLevelMode", lev::NEXT_LEVEL_NOT_BEST);
+                break;
+            case lev::NEXT_LEVEL_NOT_BEST :
+            default:
+                app.state->setProperty("NextLevelMode", lev::NEXT_LEVEL_STRICTLY);
         }
         update();
         invalidate();
     }
     
+    void AdvanceModeButton::draw(ecl::GC &gc, const ecl::Rect &r) {
+        update();
+        ImageButton::draw(gc, r);
+    }
+    
 }} // namespace enigma::gui
