@@ -23,9 +23,9 @@
 #include "gui/LevelPackConfig.hh"
 #include "gui/SearchMenu.hh"
 #include "gui/HelpMenu.hh"
-#include "ecl.hh"
 #include "errors.hh"
 #include "nls.hh"
+#include "resource_cache.hh"
 #include "video.hh"
 #include "lev/Index.hh"
 #include "lev/PersistentIndex.hh"
@@ -50,28 +50,35 @@ namespace enigma { namespace gui {
         0
     };
     
-    LevelPackMenu::LevelPackMenu() : packsHList (NULL), groupsVList (NULL),
-            scrollLeft (NULL), scrollRight (NULL), scrollUp (NULL),
-            scrollDown (NULL), isLevelMenuSubmenu (false) {
-        const video::VMInfo &vminfo = *video::GetInfo();
-        vm = vminfo.videomode;
+    LevelPackMenu::LevelPackMenu() : packsHList (NULL), scrollLeft (NULL), 
+            scrollRight (NULL), scrollUp (NULL), scrollDown (NULL), 
+            groupsVList (NULL), isLevelMenuSubmenu (false) {
+        const VMInfo &vminfo = *video_engine->GetInfo();
+        const int vshrink = vminfo.width < 640 ? 1 : 0;
+        vtt = vminfo.tt;
+        vh = vminfo.area.x;
+        vv = (vminfo.height - vminfo.area.h)/2;
        
         // Create buttons - positioning identical to Levelmenu
         but_new = new StaticTextButton(N_("New Group"), this);
         but_search = new StaticTextButton(N_("Search"), this);
         but_level = new StaticTextButton(N_("Start Game"), this);
         but_main = new StaticTextButton(N_("Main Menu"), this);
+        but_tutorial1 = NULL;
+        but_tutorial2 = NULL;
         
         commandHList = new HList;
-        commandHList->set_spacing(10);
+        commandHList->set_spacing(vshrink ? 5 : 10);
         commandHList->set_alignment(HALIGN_CENTER, VALIGN_TOP);
-        commandHList->set_default_size(140, 35);
+        commandHList->set_default_size(vshrink ? 70 : 140, vshrink ? 17 : 35);
         commandHList->add_back(but_new);
         commandHList->add_back(but_search);
         commandHList->add_back(but_level);
         commandHList->add_back(but_main);
-        this->add(commandHList, Rect(10, vminfo.height-50, vminfo.width-20, 35));
-        
+        this->add(commandHList, Rect(vshrink ? 5 : 10,
+                                     vminfo.height-(vshrink ? 25 : 50),
+                                     vminfo.width-(vshrink ? 10 : 20),
+                                     vminfo.height < 320 ? 17 : 35));
     }
     
     void LevelPackMenu::setupMenu() {
@@ -79,29 +86,62 @@ namespace enigma { namespace gui {
             int packcolumns, rows;
             int vmargin, vrow_row;
             int hmargin, hgroup_pack, hscrollbutton, hscroll_pack, hpack_pack;
-        } param[video::VM_COUNT] = {
-            {  // VM_640x480
+            int tut_rows, tut_upperrowlength, tut_upperrowheight, tut_voffset;
+        } param[] = {
+            {  // VTS_16 (320x240)
+                2, 9,
+                7, 5,
+                10, 18, 11, 5, 10,
+                1, 205, 10, 50
+            },
+            {  // VTS_32 (640x480)
                 2, 9,
                 15, 10,
-                20, 36, 22, 10, 20
+                20, 36, 22, 10, 20,
+                1, 406, 20, 100
             },
-            {  // VM_640x512
-                2, 9,
-                15, 10,
-                20, 36, 22, 10, 20
-            },
-            {  // VM_800x600
+            {  // VTS_40 (800x600)
                 3, 11,
                 15, 13,
-                15, 36, 22, 10, 15
+                15, 36, 22, 10, 15,
+                1, 576, 20, 150
             },
-            {  // VM_1024x768
-                4, 15,
+            {  // VTS_48 (960x720)  VM_1024x768
+                3, 14,
                 15, 10,
-                30, 36, 22, 12, 20
+                70, 56, 22, 20, 20,
+                3, 606, 20, 200
+            },
+            {  // VTS_64 (1280x960)
+                5, 17,
+                25, 14,
+                60, 40, 22, 18, 20,
+                7, 962, 20, 300
             }
         };
-        
+
+        static const char *tutTextLong[] = {
+// TRANSLATORS: This is the longer of two versions, used on larger screens. Your translation can be longer.
+            N_("Welcome to Enigma! Please note that the game makes use of your mouse in another way than usual, and there will be no mouse cursor during the game. You can press the ESC key or the middle mouse button to pause the game and exit (or continue). You can also always press the F1 key to see available shortcut keys."),
+// TRANSLATORS: This is the longer of two versions, used on larger screens. Your translation can be longer.
+            N_("If you are an experienced gamer and prefer a challenge for a start, please click here:"),
+// TRANSLATORS: This is the longer of two versions, used on larger screens. Your translation can be longer.
+            N_("If you prefer a more gentle introduction, please click here:"),
+// TRANSLATORS: This is the longer of two versions, used on larger screens. Your translation can be longer.
+            N_("If you feel you need some more training with handling your mouse, you might find some helpful landscapes here:"),
+        };
+
+        static const char *tutTextShort[] = {
+// TRANSLATORS: This is the shorter of two versions, used on smaller screens. Please try to keep it short.
+            N_("Welcome to Enigma! Please note that there will be no mouse cursor during the game. You can press ESC or the middle mouse button to pause or exit. Use F1 to see available shortcut keys."),
+// TRANSLATORS: This is the shorter of two versions, used on smaller screens. Please try to keep it short.
+            N_("If you are an experienced gamer and prefer a challenge for a start, please click here:"),
+// TRANSLATORS: This is the shorter of two versions, used on smaller screens. Please try to keep it short.
+            N_("For a more gentle introduction, please click here:"),
+// TRANSLATORS: This is the shorter of two versions, used on smaller screens. Please try to keep it short.
+            N_("You will find more training landscapes here:"),
+        };
+
         if (groupsVList != NULL) {
             groupsVList->clear();
             remove_child(groupsVList);
@@ -117,7 +157,7 @@ namespace enigma { namespace gui {
             delete packsHList;
             packsHList = NULL;
         }
-        
+
         if (scrollLeft != NULL) {
             remove_child(scrollLeft);
             delete scrollLeft;
@@ -126,18 +166,34 @@ namespace enigma { namespace gui {
         if (scrollRight != NULL) {
             remove_child(scrollRight);
             delete scrollRight;
-            scrollRight =  NULL;
+            scrollRight = NULL;
         }
-        
+
+        if (but_tutorial1 != NULL) {
+            remove_child(but_tutorial1);
+            delete but_tutorial1;
+            but_tutorial1 = NULL;
+        }
+        if (but_tutorial2 != NULL) {
+            remove_child(but_tutorial2);
+            delete but_tutorial1;
+            but_tutorial2 = NULL;
+        }
+
+        for (auto it = tutorialLines.begin(); it != tutorialLines.end(); it++)
+            if (*it != NULL)
+                remove_child(*it);
+        tutorialLines.clear();
+
         packButtons.clear();
         groupButtons.clear();
-        
+
         std::vector<std::string> groupNames = lev::Index::getGroupNames();
         int groupCount = groupNames.size();
         std::string curGroupName = lev::Index::getCurrentGroup();
         bool needUpScroll = false;
         bool needDownScroll = false;
-        int numDisplayGroups = param[vm].rows;
+        int numDisplayGroups = param[vtt].rows;
         int usedGroupRows = (groupCount > numDisplayGroups) ? numDisplayGroups : groupCount;
         
         // correct scroll attempts and screen resolution changes
@@ -172,12 +228,16 @@ namespace enigma { namespace gui {
                 }
             }
         }
-        
-        
+
+        // ---- build list of groups on the left side of the screen ----
+
+        int buttonWidth = (vtt==0) ? 80 : 160;
+        int buttonHeight = (vtt==0) ? 17 : 35;
+
         groupsVList = new VList; 
-        groupsVList->set_spacing(param[vm].vrow_row);
+        groupsVList->set_spacing(param[vtt].vrow_row);
         groupsVList->set_alignment(HALIGN_LEFT, VALIGN_CENTER);
-        groupsVList->set_default_size(160, 35);
+        groupsVList->set_default_size(buttonWidth, buttonHeight);
         
         for (int i = 0; i < usedGroupRows; i++) {
             if (i == 0 && needUpScroll) {
@@ -194,73 +254,145 @@ namespace enigma { namespace gui {
             }
         }
 
-        this->add(groupsVList, Rect(param[vm].hmargin, param[vm].vmargin, 
-                160, param[vm].rows * 35 + 
-                (param[vm].rows - 1) * param[vm].vrow_row));
-        
+        this->add(groupsVList, Rect(param[vtt].hmargin + vh, param[vtt].vmargin + vv,
+                buttonWidth, param[vtt].rows * buttonHeight +
+                (param[vtt].rows - 1) * param[vtt].vrow_row));
+
         lastGroupName = curGroupName;
+
+        // ---- build list of level packs of current group in the middle of the screen ----
+
         std::vector<lev::Index *> * group = lev::Index::getGroup(curGroupName);
-        ASSERT(group != NULL, XFrontend,"");
+        ASSERT(group != NULL, XFrontend, "curGroupName corresponds to NULL group");
         unsigned packCount = group->size();
-        
+        int packOffset = 0;
         int posCurrentIndex = getIndexPosition(group, lev::Index::getCurrentIndex()->getName());
         int selectedColumn = lev::Index::getGroupSelectedColumn(curGroupName);
         int colCurrentIndex = 0;
         int nextPack = 0;        // pack displayed at top of first display column
-        
+        int numRows = param[vtt].rows;
+        int vOffset = param[vtt].vmargin + vv;
+
+        if ((curGroupName == INDEX_TUTORIALS_GROUP) && (packCount >= 2)) {
+            std::string workString;
+            std::vector<std::string> lines;
+            ecl::Font *menufont = enigma::GetFont("menufont");
+            int x = param[vtt].hmargin + vh + buttonWidth + param[vtt].hgroup_pack;
+            int y = vOffset + param[vtt].tut_voffset;
+            int w = param[vtt].tut_upperrowlength;
+            int h = param[vtt].tut_upperrowheight;
+            int v_text_button = (vtt == 0) ? 8 : 16;
+            // welcome text and information about ESC and F1
+            workString = (vtt <= 1) ? tutTextShort[0] : tutTextLong[0];
+            lines = breakToLines(menufont, _(workString), " ", param[vtt].tut_upperrowlength);
+            for (auto it = lines.begin(); it != lines.end(); it++) {
+                Label * lbText = new UntranslatedLabel(*it, HALIGN_LEFT);
+                this->add(lbText, Rect(x, y, w, h));
+                tutorialLines.push_back(lbText);
+                y += h+1;
+            }
+            y += 4;
+#ifdef ENABLE_EXPERIMENTAL
+            // recommendation for advanced tutorial pack
+            workString = (vtt <= 1) ? tutTextShort[1] : tutTextLong[1];
+            lines = breakToLines(menufont, _(workString), " ", param[vtt].tut_upperrowlength);
+            for (auto it = lines.begin(); it != lines.end(); it++) {
+                Label * lbText = new UntranslatedLabel(*it, HALIGN_LEFT);
+                this->add(lbText, Rect(x, y, w, h));
+                tutorialLines.push_back(lbText);
+                y += h+1;
+            }
+            y += v_text_button;
+            // button for advanced tutorial pack
+            lev::Index *ind1 = (*group)[0];
+            but_tutorial1 = new StaticTextButton(ind1->getName(), this);
+            packButtons.push_back(but_tutorial1);
+            this->add(but_tutorial1, Rect(x + (w - buttonWidth)/2 - 1, y, buttonWidth, buttonHeight));
+            y += buttonHeight + v_text_button;
+            // recommendation for basic tutorial pack
+            workString = (vtt <= 1) ? tutTextShort[2] : tutTextLong[2];
+            lines = breakToLines(menufont, _(workString), " ", param[vtt].tut_upperrowlength);
+            for (auto it = lines.begin(); it != lines.end(); it++) {
+                Label * lbText = new UntranslatedLabel(*it, HALIGN_LEFT);
+                this->add(lbText, Rect(x, y, w, h));
+                tutorialLines.push_back(lbText);
+                y += h+1;
+            }
+            y += v_text_button;
+            // button for basic tutorial pack
+            lev::Index *ind2 = (*group)[1];
+            but_tutorial2 = new StaticTextButton(ind2->getName(), this);
+            packButtons.push_back(but_tutorial2);
+            this->add(but_tutorial2, Rect(x + (w - buttonWidth)/2 - 1, y, buttonWidth, buttonHeight));
+            y += buttonHeight + v_text_button;
+            // recommendation for remaining level pack(s)
+            workString = (vtt <= 1) ? tutTextShort[3] : tutTextLong[3];
+            lines = breakToLines(menufont, _(workString), " ", param[vtt].tut_upperrowlength);
+            for (auto it = lines.begin(); it != lines.end(); it++) {
+                Label * lbText = new UntranslatedLabel(*it, HALIGN_LEFT);
+                this->add(lbText, Rect(x, y, w, h));
+                tutorialLines.push_back(lbText);
+                y += h+1;
+            }
+#endif
+            numRows = param[vtt].tut_rows;
+            vOffset = y + v_text_button;
+            packCount -= 2;
+            packOffset = 2;
+        }
+
         if (selectedColumn != INDEX_GROUP_COLUMN_UNKNOWN || 
                 groupLastSelectedIndex.find(curGroupName) == groupLastSelectedIndex.end()) {
-            colCurrentIndex = checkColumn(param[vm].rows, param[vm].packcolumns,
+            colCurrentIndex = checkColumn(numRows, param[vtt].packcolumns,
                     packCount, posCurrentIndex, selectedColumn);
-            nextPack = (posCurrentIndex / param[vm].rows - colCurrentIndex) * param[vm].rows;
+            nextPack = (posCurrentIndex / numRows - colCurrentIndex) * numRows;
         } else {
             // the user selected a new level pack and the column was not yet
             // calculated: we try to keep the display unchanged in respect of
             // of the last selected pack and if necessary scroll minimum amount
             // of columns 
             int posLastIndex = getIndexPosition(group,groupLastSelectedIndex[curGroupName]);
-            int colLastIndex = checkColumn(param[vm].rows, param[vm].packcolumns,
+            int colLastIndex = checkColumn(numRows, param[vtt].packcolumns,
                     packCount, posLastIndex, groupLastSelectedColumn[curGroupName]);
-            nextPack = (posLastIndex / param[vm].rows - colLastIndex) * param[vm].rows;
+            nextPack = (posLastIndex / numRows - colLastIndex) * numRows;
             if (posCurrentIndex < nextPack) {
                 // current index would be left of display - we need to scroll left
-                nextPack -= (((nextPack - posCurrentIndex - 1)/param[vm].rows) + 1) *
-                        param[vm].rows;
+                nextPack -= (((nextPack - posCurrentIndex - 1)/numRows) + 1) * numRows;
                 colCurrentIndex = 0;
-            } else if (posCurrentIndex < nextPack + param[vm].rows * param[vm].packcolumns) {
+            } else if (posCurrentIndex < nextPack + numRows * param[vtt].packcolumns) {
                 // current index is still visible - keep nextPack
-                colCurrentIndex = (posCurrentIndex - nextPack) / param[vm].rows;
+                colCurrentIndex = (posCurrentIndex - nextPack) / numRows;
             } else {
                 // current index would be right of display - we need to scroll right
-                nextPack += (((posCurrentIndex - nextPack)/param[vm].rows) - 
-                        (param[vm].packcolumns - 1)) * param[vm].rows;
-                colCurrentIndex = param[vm].packcolumns - 1;                
+                nextPack += (((posCurrentIndex - nextPack)/numRows) -
+                        (param[vtt].packcolumns - 1)) * numRows;
+                colCurrentIndex = param[vtt].packcolumns - 1;                
             }
         }
         
-        bool needRightScroll = packCount > nextPack + param[vm].rows * param[vm].packcolumns;
-        bool needLeftScroll = nextPack > 0;
+        bool needRightScroll = ((int)packCount > nextPack + numRows * param[vtt].packcolumns);
+        bool needLeftScroll = (nextPack > 0);
 
         lev::Index::setGroupSelectedColumn(curGroupName, colCurrentIndex);
         groupLastSelectedIndex[curGroupName] = lev::Index::getCurrentIndex()->getName();
         groupLastSelectedColumn[curGroupName] = colCurrentIndex;
         
         packsHList = new HList; 
-        packsHList->set_spacing(param[vm].hpack_pack);
+        packsHList->set_spacing(param[vtt].hpack_pack);
         packsHList->set_alignment(HALIGN_CENTER, VALIGN_TOP);
-        packsHList->set_default_size(160, param[vm].rows*35 + 
-                (param[vm].rows - 1) * param[vm].vrow_row);
-        
-        for (int col = 0; col < param[vm].packcolumns; col++) {
+        packsHList->set_default_size(buttonWidth, numRows * buttonHeight +
+                (numRows - 1) * param[vtt].vrow_row);
+
+        for (int col = 0; col < param[vtt].packcolumns; col++) {
             if (packCount - nextPack > 0) {
                 VList * pl = new VList;
-                pl->set_spacing (param[vm].vrow_row);
+                pl->set_spacing (param[vtt].vrow_row);
                 // first column is centered - if it is full it is like top alignment:
                 pl->set_alignment (HALIGN_LEFT, col == 0 ? VALIGN_CENTER : VALIGN_TOP);
-                pl->set_default_size (160, 35);
-                for (int row = 0; row < param[vm].rows; row++) {
+                pl->set_default_size (buttonWidth, buttonHeight);
+                for (int row = 0; row < numRows; row++) {
                     if (nextPack < packCount) {
-                        lev::Index *ind = (*group)[nextPack];
+                        lev::Index *ind = (*group)[nextPack + packOffset];
                         TextButton * button = new UntranslatedStaticTextButton(ind->getName(), this);
                         packButtons.push_back(button);
                         pl->add_back(button);
@@ -268,37 +400,35 @@ namespace enigma { namespace gui {
                     } else
                         break;
                 }
-                packsHList->add_back(pl);               
+                packsHList->add_back(pl);
             } else
                 break;
         }
 
-        this->add(packsHList, Rect(param[vm].hmargin + 160 + param[vm].hgroup_pack +
-                param[vm].hscrollbutton + param[vm].hscroll_pack, 
-                param[vm].vmargin, 
-                param[vm].packcolumns * 160 + (param[vm].packcolumns - 1) * 
-                param[vm].hpack_pack, 
-                param[vm].rows * 35 + 
-                (param[vm].rows - 1) * param[vm].vrow_row));
+        this->add(packsHList, Rect(param[vtt].hmargin + vh + buttonWidth + param[vtt].hgroup_pack +
+                param[vtt].hscrollbutton + param[vtt].hscroll_pack,
+                vOffset,
+                param[vtt].packcolumns * buttonWidth + (param[vtt].packcolumns - 1) *
+                param[vtt].hpack_pack,
+                numRows * buttonHeight + (numRows - 1) * param[vtt].vrow_row));
         
         if (needLeftScroll) {
             scrollLeft = new ImageButton("ic-left", "ic-left1", this);
-            this->add(scrollLeft, Rect(param[vm].hmargin + 160 + param[vm].hgroup_pack,
-                    param[vm].vmargin + param[vm].rows / 2 * (35 + param[vm].vrow_row),
-                    param[vm].hscrollbutton, 35));
+            this->add(scrollLeft, Rect(param[vtt].hmargin + vh + buttonWidth + param[vtt].hgroup_pack,
+                    vOffset + numRows / 2 * (buttonHeight + param[vtt].vrow_row),
+                    param[vtt].hscrollbutton, buttonHeight));
         }
                 
         if (needRightScroll) {
             scrollRight = new ImageButton("ic-right", "ic-right1", this);
-            this->add(scrollRight, Rect(param[vm].hmargin + 160 + param[vm].hgroup_pack +
-                    param[vm].hscrollbutton + 2 * param[vm].hscroll_pack +
-                    param[vm].packcolumns * 160 + (param[vm].packcolumns - 1) * 
-                    param[vm].hpack_pack,
-                    param[vm].vmargin + param[vm].rows / 2 * (35 + param[vm].vrow_row),
-                    param[vm].hscrollbutton, 35));
+            this->add(scrollRight, Rect(param[vtt].hmargin + vh + buttonWidth + param[vtt].hgroup_pack +
+                    param[vtt].hscrollbutton + 2 * param[vtt].hscroll_pack +
+                    param[vtt].packcolumns * buttonWidth + (param[vtt].packcolumns - 1) * param[vtt].hpack_pack,
+                    vOffset + numRows / 2 * (buttonHeight + param[vtt].vrow_row),
+                    param[vtt].hscrollbutton, buttonHeight));
         }
     }
-    
+
     void LevelPackMenu::manageLevelMenu() {
         bool finished = false;
         while (!finished) {
@@ -332,7 +462,7 @@ namespace enigma { namespace gui {
     bool LevelPackMenu::on_event (const SDL_Event &e) {
         switch (e.type) {
             case SDL_KEYDOWN:
-                SDLKey keysym = e.key.keysym.sym;
+                SDL_Keycode keysym = e.key.keysym.sym;
                 switch (keysym) {
                 case SDLK_F1:     
                     displayHelp(helptext, 200);
@@ -357,7 +487,7 @@ namespace enigma { namespace gui {
             invalidate_all();
         } else if (w == but_level) {
             LevelMenu m;
-            if (!m.manage() && isLevelMenuSubmenu || m.isMainQuit()) {
+            if ((!m.manage() && isLevelMenuSubmenu) || m.isMainQuit()) {
                 // ESC in LevelMenu in case we are a submenu of LevelMenu or
                 // Main button has been pressed in LevelMenu 
                 Menu::quit();
@@ -371,7 +501,7 @@ namespace enigma { namespace gui {
             if (ml.isSearchQuit()) {
                 // show search result levelpack
                 LevelMenu ml;
-                if (!ml.manage() && isLevelMenuSubmenu || ml.isMainQuit()) {
+                if ((!ml.manage() && isLevelMenuSubmenu) || ml.isMainQuit()) {
                     // ESC in LevelMenu in cade we are a submenu of LevelMenu or
                     // Main button has been pressed in LevelMenu
                     Menu::quit();
@@ -407,7 +537,9 @@ namespace enigma { namespace gui {
             setupMenu();
             updateHighlight();
             invalidate_all();
-        } else if (w->get_parent()->get_parent() == packsHList){
+        } else if (   (w->get_parent()->get_parent() == packsHList)
+                   || (w == but_tutorial1)
+                   || (w == but_tutorial2)) {
             lev::Index::setCurrentIndex(dynamic_cast<TextButton *>(w)->get_text());
             if (w->lastMouseButton() == SDL_BUTTON_RIGHT ||
                      w->lastModifierKeys() & KMOD_CTRL) {
@@ -416,7 +548,7 @@ namespace enigma { namespace gui {
                 m.manage();
             } else {
                 LevelMenu m;
-                if (!m.manage() && isLevelMenuSubmenu || m.isMainQuit()) {
+                if ((!m.manage() && isLevelMenuSubmenu) || m.isMainQuit()) {
                     // ESC in LevelMenu in case we are a submenu of LevelMenu or
                     // Main button has been pressed in LevelMenu
                     Menu::quit();
@@ -447,14 +579,14 @@ namespace enigma { namespace gui {
     }
     
     void LevelPackMenu::updateHighlight() {
-        for (int i = 0; i < packButtons.size(); i++) {
+        for (unsigned i = 0; i < packButtons.size(); i++) {
             TextButton * button = packButtons[i];
             if (button->get_text() == lev::Index::getCurrentIndex()->getName())
                 button->setHighlight(true);
             else
                 button->setHighlight(false);      
         }
-        for (int i = 0; i < groupButtons.size(); i++) {
+        for (unsigned i = 0; i < groupButtons.size(); i++) {
             TextButton * button = groupButtons[i];
             if (button->get_text() == lev::Index::getCurrentGroup())
                 button->setHighlight(true);
@@ -464,8 +596,9 @@ namespace enigma { namespace gui {
     }
     
     void LevelPackMenu::draw_background(ecl::GC &gc) {
-        video::SetCaption(("Enigma - Level Pack Menu"));
-        blit(gc, 0,0, enigma::GetImage("menu_bg", ".jpg"));
+        const VMInfo *vminfo = video_engine->GetInfo();
+        set_caption(_("Enigma - Level Pack Menu"));
+        blit(gc, vminfo->mbg_offsetx, vminfo->mbg_offsety, enigma::GetImage("menu_bg", ".jpg"));
     }
     
     int LevelPackMenu::getGroupPosition(std::vector<std::string> * groups, std::string groupName) {
