@@ -22,33 +22,36 @@
 #include "gui/LevelMenu.hh"
 #include "gui/LevelInspector.hh"
 
-#include "ecl.hh"
+#include "ecl_font.hh"
 #include "main.hh"
 #include "nls.hh"
 #include "options.hh"
-#include "sound.hh"
+#include "resource_cache.hh"
+#include "SoundEffectManager.hh"
 #include "StateManager.hh"
 #include "video.hh"
-#include "file.hh"
 #include "lev/Proxy.hh"
 #include <cassert>
 
 using namespace ecl;
 using namespace std;
 
+#define SCREEN ecl::Screen::get_instance()
 
-namespace enigma { namespace gui {    
+namespace enigma { namespace gui {
     /* -------------------- LevelWidget -------------------- */
     
     LevelWidget::LevelWidget(bool withScoreIcons, bool withEditBorder) : 
             displayScoreIcons (withScoreIcons), displayEditBorder (withEditBorder),
-            width (0), height (0), m_areas(),
-            listener(0), isInvalidateUptodate (true), lastUpdate (0)
+            listener(0), width (0), height (0), m_areas(), 
+            isInvalidateUptodate (true), lastUpdate (0)
     {
-        const video::VMInfo &vminfo = *video::GetInfo();
-    
-        buttonw = vminfo.thumbw + 20;
-        buttonh = vminfo.thumbh + 28;
+        const VMInfo &vminfo = *video_engine->GetInfo();
+        const int vshrink = vminfo.width < 640 ? 1 : 0;
+
+        buttonw = vminfo.thumb.width +
+                  (vshrink ? 13 : 27);  // min should be +30 for all modes but 640x480
+        buttonh = vminfo.thumb.height + (vshrink ? 22 : ((vminfo.height==960)?35:44));
         curIndex = lev::Index::getCurrentIndex();
         iselected = curIndex->getCurrentPosition();
         ifirst = curIndex->getScreenFirstPosition();
@@ -59,14 +62,15 @@ namespace enigma { namespace gui {    
         img_feather     = enigma::GetImage("ic-feather");
         img_easy        = enigma::GetImage("completed-easy");
         img_hard        = enigma::GetImage("completed");
-        img_changed     = enigma::GetImage("changed");
+        img_obsolete = enigma::GetImage(("ic-obsolete" + vminfo.thumb.suffix).c_str());
+        img_outdated = enigma::GetImage(("ic-outdated" + vminfo.thumb.suffix).c_str());
         img_unavailable = enigma::GetImage("unavailable");
-//        img_unknown     = enigma::GetImage("unknown");
         img_par         = enigma::GetImage("par");
         img_wrEasy      = enigma::GetImage("ic-wr-easy");
         img_wrDifficult = enigma::GetImage("ic-wr-difficult");
-        img_border      = enigma::GetImage("thumbborder");
-        img_editborder  = enigma::GetImage("editborder");
+        img_border = enigma::GetImage(("thumbborder" + vminfo.thumb.suffix).c_str());
+        img_editborder = enigma::GetImage(("editborder" + vminfo.thumb.suffix).c_str());
+        thumbmode = (vminfo.thumb.width == 160) ? 2 : ((vminfo.thumb.width == 120) ? 1 : 0);
     }
     
     void LevelWidget::syncFromIndexMgr() {
@@ -89,9 +93,8 @@ namespace enigma { namespace gui {    
         curIndex->setScreenFirstPosition(ifirst);
     }
         
-    void LevelWidget::realize (const ecl::Rect &area_)
-    {
-        Widget::realize (area_);
+    void LevelWidget::realize(const ecl::Rect &area_) {
+        Widget::realize(area_);
         width = area_.w / buttonw;
         height = area_.h / buttonh;
     }
@@ -102,7 +105,7 @@ namespace enigma { namespace gui {    
         }
     }
         
-    void LevelWidget::scroll_up (int nlines) 
+    void LevelWidget::scroll_up(int nlines) 
     {
         int newFirst = ifirst;
         int newSelected = iselected;
@@ -113,7 +116,7 @@ namespace enigma { namespace gui {    
             if (newSelected < newFirst)
                 newSelected += width;
         }
-        set_selected (newFirst, newSelected);
+        set_selected(newFirst, newSelected);
     }
     
     void LevelWidget::scroll_down(int nlines) 
@@ -133,11 +136,11 @@ namespace enigma { namespace gui {    
                     newSelected -= width;
             }
         }
-        set_selected (newFirst, newSelected);
+        set_selected(newFirst, newSelected);
     }
     
     void LevelWidget::page_up() {
-        set_selected ((ifirst >= width*height ? ifirst - width*height : 0), 
+        set_selected((ifirst >= width*height ? ifirst - width*height : 0), 
                 (iselected >= width*height ? iselected - width*height : 0));
         syncToIndexMgr();
     }
@@ -150,7 +153,7 @@ namespace enigma { namespace gui {    
         // make sure last page is shown as a whole
         int first = std::min<int> (lastPageFirst, ifirst + width*height);
         //    set_selected (first, s-1);
-        set_selected (first, iselected + width*height);
+        set_selected(first, iselected + width*height);
         syncToIndexMgr();
     }
     
@@ -166,12 +169,12 @@ namespace enigma { namespace gui {    
         syncToIndexMgr();
     }
     
-    void LevelWidget::set_current (int newsel)
+    void LevelWidget::set_current(int newsel)
     {
-        set_selected (ifirst, newsel);
+        set_selected(ifirst, newsel);
     }
     
-    void LevelWidget::set_selected (int newfirst, int newsel)
+    void LevelWidget::set_selected(int newfirst, int newsel)
     {
         int numlevels = curIndex->size();
         newsel = Clamp<int> (newsel, 0, numlevels-1);
@@ -182,7 +185,7 @@ namespace enigma { namespace gui {    
         if (newsel >= newfirst+width*height)
             newfirst = (newsel/width-height+1)*width;
     
-        newfirst = Clamp<int> (newfirst, 0, numlevels-1);
+        newfirst = ecl::Clamp<int>(newfirst, 0, numlevels-1);
         if (newfirst < 0) newfirst = 0;
     
         size_t oldsel = iselected;
@@ -192,7 +195,7 @@ namespace enigma { namespace gui {    
     
             if (!m_areas.empty()) {
                 sound::EmitSoundEvent ("menumove");
-                if (oldsel != newsel) 
+                if ((int)oldsel != newsel) 
                     sound::EmitSoundEvent ("menuswitch");
                 invalidate();
             }
@@ -208,23 +211,31 @@ namespace enigma { namespace gui {    
         }
     }
                 
-    bool LevelWidget::draw_level_preview (ecl::GC &gc, int x, int y, 
+    int LevelWidget::thumb_off(int small, int medium, int large) {
+    	return (thumbmode == 2) ? large : ((thumbmode == 1) ? medium : small);
+    }
+
+    bool LevelWidget::draw_level_preview(ecl::GC &gc, int x, int y, int borderWidth,
             lev::Proxy *proxy, bool selected, bool isCross, bool locked,
             bool allowGeneration, bool &didGenerate) { 
         // Draw button with level preview
-    
-        Surface *img = preview_cache->getPreview(proxy, allowGeneration, didGenerate);
+
+        const VMInfo *vminfo = video_engine->GetInfo();
+        Surface *img =
+            preview_cache->getPreview(proxy, vminfo->thumb, allowGeneration, didGenerate);
         if (img == NULL)
             return false;
    
         if (selected) {
-            blit (gc, x-4, y-4, displayEditBorder ? img_editborder : img_border);
+            blit (gc, x - borderWidth, y - borderWidth, displayEditBorder ? img_editborder : img_border);
             blit (gc, x, y, img);
-        }
-        else {
-            img->set_alpha (127);
+        } else {
+            // With SDL2, set_alpha doesn't work properly anymore. We are therefore
+            // using set_brightness instead.
+            // TODO: Revert to set_alpha, once it's repaired.
+            img->set_brightness(170);
             blit (gc, x, y, img);
-            img->set_alpha(255);
+            img->set_brightness(255);
         }
 
         // Shade unavailable levels
@@ -242,7 +253,7 @@ namespace enigma { namespace gui {    
             //   Silver + Gold: Level beaten in all modes - easy available
             Surface *useAsEasy = NULL;
             Surface *useAsDifficult = NULL;
-            if (proxy->hasEasymode()) {
+            if (proxy->hasEasyMode()) {
                 useAsEasy = img_feather;
                 if (scoreMgr->isSolved(proxy, DIFFICULTY_EASY))
                     useAsEasy = img_easy;
@@ -253,29 +264,31 @@ namespace enigma { namespace gui {    
             if (app.state->getInt("Difficulty") == DIFFICULTY_HARD) {
                 // draw golden medal over silber medal
                 if (useAsEasy != NULL)
-                    blit (gc, x, y, useAsEasy);
+                    blit (gc, x+thumb_off(3,3,24), y, useAsEasy);
                 if (useAsDifficult != NULL)
-                    blit (gc, x+5, y, useAsDifficult);
+                    blit (gc, x+thumb_off(8,8,29), y, useAsDifficult);
             }
             else {
                 // draw silver medal over golden medal
                 if (useAsDifficult != NULL)
-                    blit (gc, x+5, y, useAsDifficult);
+                    blit (gc, x+thumb_off(8,8,29), y, useAsDifficult);
                 if (useAsEasy != NULL)
-                    blit (gc, x, y, useAsEasy);
+                    blit (gc, x+thumb_off(3,3,24), y, useAsEasy);
             }
         
             // Add warning sign if level has been changed since player solved it
-            if (scoreMgr->isOutdated(proxy, app.state->getInt("Difficulty")))
-                blit (gc, x-3, y-3, img_changed);
+            if (scoreMgr->isObsolete(proxy, app.state->getInt("Difficulty")))
+                blit(gc, x-2, y-2, img_obsolete);
+            else if (scoreMgr->isOutdated(proxy, app.state->getInt("Difficulty")))
+                blit(gc, x-2, y-2, img_outdated);
         
             // Add icon if worldrecord or par
             if (scoreMgr->bestScoreReached(proxy, app.state->getInt("Difficulty"))) {
-                blit(gc, x+35, y+5, 
+                blit(gc, x+thumb_off(5,35,59), y+thumb_off(2,5,20),
                         (app.state->getInt("Difficulty") != DIFFICULTY_HARD &&
-                        proxy->hasEasymode()) ? img_wrEasy : img_wrDifficult);
+                        proxy->hasEasyMode()) ? img_wrEasy : img_wrDifficult);
             } else if (scoreMgr->parScoreReached(proxy, app.state->getInt("Difficulty"))){
-                blit(gc, x+30, y+12, img_par);
+                blit(gc, x+thumb_off(33,33,55), y+thumb_off(12,12,12), img_par);
             }
         } else {
             // Draw solved/changed icons on top of level preview
@@ -287,23 +300,21 @@ namespace enigma { namespace gui {    
         return true;
     }
     
-    void LevelWidget::draw (ecl::GC &gc, const ecl::Rect &r)
-    {
-        const video::VMInfo &vminfo = *video::GetInfo();
-        const int imgw = vminfo.thumbw;       // Size of the preview images
-        const int imgh = vminfo.thumbh;
-    
+    void LevelWidget::draw(ecl::GC &gc, const ecl::Rect &r) {
+        const VMInfo &vminfo = *video_engine->GetInfo();
+        const int imgw = vminfo.thumb.width;  // Size of the preview images
+        const int imgh = vminfo.thumb.height;
+        const int bwidth = vminfo.thumb.border_width;
+
         const int hgap = Max(0, (get_w() - width*buttonw) / (width));
         const int vgap = Max(0, (get_h() - height*buttonh)/ (height-1));
     
         unsigned i=ifirst;          // level index
         bool allowGeneration = true;
     
-        for (int y=0; y<height; y++)
-        {
-            for (int x=0; x<width; x++, i++)
-            {
-                if (i >= curIndex->size())
+        for (int y=0; y<height; y++) {
+            for (int x=0; x<width; x++, i++) {
+                if ((int)i >= curIndex->size())
                     goto done_painting;
     
                 int xpos = get_x() + hgap/2 + x*(buttonw + hgap);
@@ -321,12 +332,12 @@ namespace enigma { namespace gui {    
                 }
                 // Draw level preview
                 lev::Proxy *levelProxy = curIndex->getProxy(i);
-                int imgx = xpos+(buttonw-imgw)/2;
-                int imgy = ypos + 4;
+                int imgx = xpos + (buttonw-imgw)/2;
+                int imgy = ypos + bwidth;
                 if (levelProxy != NULL) {
                     bool didGenerate;
-                    bool didDraw = draw_level_preview (gc, imgx, imgy, levelProxy, 
-                            i == iselected, !curIndex->isSource(levelProxy), 
+                    bool didDraw = draw_level_preview(gc, imgx, imgy, bwidth, levelProxy, 
+                            (int) i == iselected, !curIndex->isSource(levelProxy), 
                             !curIndex->mayPlayLevel(i+1),
                             allowGeneration, didGenerate);
                     if (didGenerate) {
@@ -342,27 +353,35 @@ namespace enigma { namespace gui {    
                         pending_redraws[(i-ifirst)] = true;
                         isInvalidateUptodate = false;
                     }
+                    // Draw level name
+                    Font *smallfnt = enigma::GetFont("levelmenu");
+                    Font *altsmallfnt = enigma::GetFont("smallalternative");;
+                    int maxwidth = buttonw + hgap;
+                    std::string caption = levelProxy->getTitle();
+                    std::vector<std::string> lines = ecl::breakToLines(smallfnt, caption, " ", maxwidth);
+                    if ((lines.size() > 0) && !lines[0].empty())
+                        smallfnt->render (gc,
+                              xpos + buttonw/2 - ecl::Min(smallfnt->get_width(lines[0].c_str(), altsmallfnt)/2, maxwidth/2),
+                              imgy + imgh + ((lines.size() == 1) ? 2 : 0),
+                              lines[0], altsmallfnt, maxwidth);
+                    if ((vminfo.height != 960) && (lines.size() > 1) && !lines[1].empty())
+                        smallfnt->render (gc,
+                              xpos + buttonw/2 - ecl::Min(smallfnt->get_width(lines[1].c_str(), altsmallfnt)/2, maxwidth/2),
+                              imgy + imgh + smallfnt->get_height() - 3,
+                              lines[1], altsmallfnt, maxwidth);
                 }
-                // Draw level name
-                Font    *smallfnt = enigma::GetFont("levelmenu");
-                Font    *altsmallfnt = enigma::GetFont("smallalternative");;
-                std::string caption = levelProxy->getTitle();
-                smallfnt->render (gc,
-                          xpos + buttonw/2 - ecl::Min(smallfnt->get_width(caption.c_str(), altsmallfnt)/2, (buttonw+hgap)/2),
-                          imgy + imgh + 2,
-                          caption, altsmallfnt, buttonw + hgap);
             }
         }
         done_painting:
-        m_areas.resize (i-ifirst); // Remove unused areas (if any) from the list
+        m_areas.resize(i-ifirst); // Remove unused areas (if any) from the list
         return;
     }
     
-    void LevelWidget::tick (double time) {
+    void LevelWidget::tick(double time) {
         if (!isInvalidateUptodate) {
             // invalidate just 1 button for redraw
             bool isFirst = true;
-            for (int i = 0; i < pending_redraws.size(); i++) {
+            for (unsigned i = 0; i < pending_redraws.size(); i++) {
                 if (pending_redraws[i] == true) {
                     if (isFirst) {
                         invalidate_area(m_areas[i]);
@@ -377,112 +396,125 @@ namespace enigma { namespace gui {    
         }
     }
     
-    bool LevelWidget::on_event(const SDL_Event &e) 
-    {
+    bool LevelWidget::on_event(const SDL_Event &e) {
         bool handled = Widget::on_event(e);
+        int mouse_x = 0;
+        int mouse_y = 0;
     
         switch (e.type) {
-        case SDL_MOUSEMOTION:
-            if (get_area().contains(e.motion.x, e.motion.y)) {
-                int newsel=iselected;
-                for (unsigned i=0; i<m_areas.size(); ++i)
-                    if (m_areas[i].contains(e.motion.x, e.motion.y))
-                    {
-                        newsel = ifirst+i;
-                        break;
-                    }
-                set_current(newsel);
-                handled = true;
-            }
-            break;
-        case SDL_MOUSEBUTTONDOWN:
-            if (get_area().contains(e.button.x, e.button.y))
-                handled = handle_mousedown (&e);
-            break;
-        case SDL_KEYDOWN:
-            handled = handle_keydown (&e);
-            break;
+            case SDL_MOUSEMOTION:
+                mouse_x = (int)((double) (e.motion.x * SCREEN->size().w) / SCREEN->window_size().w + 0.5);
+                mouse_y = (int)((double) (e.motion.y * SCREEN->size().h) / SCREEN->window_size().h + 0.5);
+                if (get_area().contains(mouse_x, mouse_y)) {
+                    int newsel=iselected;
+                    for (unsigned i=0; i<m_areas.size(); ++i)
+                        if (m_areas[i].contains(mouse_x, mouse_y))
+                        {
+                            newsel = ifirst+i;
+                            break;
+                        }
+                    set_current(newsel);
+                    handled = true;
+                }
+                break;
+            case SDL_MOUSEBUTTONDOWN:
+                mouse_x = (int)((double) (e.button.x * SCREEN->size().w) / SCREEN->window_size().w + 0.5);
+                mouse_y = (int)((double) (e.button.y * SCREEN->size().h) / SCREEN->window_size().h + 0.5);
+                if (get_area().contains(mouse_x, mouse_y))
+                    handled = handle_mousedown (&e);
+                break;
+            case SDL_MOUSEWHEEL:
+                if (e.wheel.y < 0) {
+                    scroll_up(1);
+                    handled = true;
+                }
+                if (e.wheel.y > 0) {
+                    scroll_down(1);
+                    handled = true;
+                }
+                break;
+            case SDL_KEYDOWN:
+                handled = handle_keydown (&e);
+                break;
         }
         syncToIndexMgr();
         return handled;
     }
     
-    bool LevelWidget::handle_mousedown (const SDL_Event *e) 
-    {
+    bool LevelWidget::handle_mousedown(const SDL_Event *e) {
+        int mouse_x = (int)((double) (e->button.x * SCREEN->size().w) / SCREEN->window_size().w + 0.5);
+        int mouse_y = (int)((double) (e->button.y * SCREEN->size().h) / SCREEN->window_size().h + 0.5);
         switch (e->button.button) {
-        case SDL_BUTTON_LEFT:
-            for (unsigned i=0; i<m_areas.size(); ++i)
-                if (m_areas[i].contains(e->button.x, e->button.y))
-                {
-                    sound::EmitSoundEvent ("menuok");
-                    iselected = ifirst+i;
-                    syncToIndexMgr();
-                    if (SDL_GetModState() & KMOD_CTRL && !(SDL_GetModState() & KMOD_SHIFT)) {
-                        // control key pressed - level inspector
+            case SDL_BUTTON_LEFT:
+                for (unsigned i=0; i<m_areas.size(); ++i)
+                    if (m_areas[i].contains(mouse_x, mouse_y))
+                    {
+                        sound::EmitSoundEvent ("menuok");
+                        iselected = ifirst+i;
+                        syncToIndexMgr();
+                        if (SDL_GetModState() & KMOD_CTRL && !(SDL_GetModState() & KMOD_SHIFT)) {
+                            // control key pressed - level inspector
+                            LevelInspector m(curIndex->getProxy(iselected));
+                            m.manage();
+                            get_parent()->draw_all();
+                        } else {
+                            // no control key - start level
+                            trigger_action();
+                        }
+                        return true;
+                    }
+                break;
+            case SDL_BUTTON_RIGHT: 
+                for (unsigned i=0; i<m_areas.size(); ++i)
+                    if (m_areas[i].contains(mouse_x, mouse_y))
+                    {
+                        sound::EmitSoundEvent ("menuok");
+                        iselected = ifirst+i;
+                        syncToIndexMgr();
                         LevelInspector m(curIndex->getProxy(iselected));
                         m.manage();
                         get_parent()->draw_all();
-                    } else {
-                        // no control key - start level
-                        trigger_action();
+                        return true;
                     }
-                    return true;
-                }
-            break;
-        case SDL_BUTTON_RIGHT: 
-            for (unsigned i=0; i<m_areas.size(); ++i)
-                if (m_areas[i].contains(e->button.x, e->button.y))
-                {
-                    sound::EmitSoundEvent ("menuok");
-                    iselected = ifirst+i;
-                    syncToIndexMgr();
-                    LevelInspector m(curIndex->getProxy(iselected));
-                    m.manage();
-                    get_parent()->draw_all();
-                    return true;
-                }
-            break;
-        case 4: scroll_down(1); return true;
-        case 5: scroll_up(1); return true;
+                break;
         }
         return false;
     }
     
-    bool LevelWidget::handle_keydown (const SDL_Event *e)
-    {
+    bool LevelWidget::handle_keydown(const SDL_Event *e) {
         switch (e->key.keysym.sym) {
-    
-        case SDLK_t:
-            // Generate new level preview for current level
-            preview_cache->updatePreview(curIndex->getProxy(iselected));
-            invalidate();
-            break;
-        
-        case SDLK_LEFT:  
-            if (!(SDL_GetModState() & KMOD_ALT)) {
-                set_current (iselected>1 ? iselected-1 : 0); 
+            case SDLK_t:
+                // Generate new level preview for current level
+                preview_cache->updatePreview(curIndex->getProxy(iselected),
+                                             video_engine->GetInfo()->thumb);
+                invalidate();
                 break;
-            } else
-                return false;
-        case SDLK_RIGHT:
-            if (!(SDL_GetModState() & KMOD_ALT)) {
-                 set_current (iselected+1);
-                 break;
-            } else
-                return false;
-        case SDLK_DOWN:  set_current (iselected+width); break;
-        case SDLK_UP:    set_current (iselected>width ? iselected-width : 0); break;
-        case SDLK_PAGEDOWN: page_down(); break;
-        case SDLK_PAGEUP: page_up(); break;
-        case SDLK_HOME: start(); break;
-        case SDLK_END: end(); break;
-    
-        case SDLK_RETURN:
-            trigger_action();
-            break;
-    
-        default:
-            return false;           // key not handled
+            
+            case SDLK_LEFT:  
+                if (!(SDL_GetModState() & KMOD_ALT)) {
+                    set_current (iselected>1 ? iselected-1 : 0); 
+                    break;
+                } else
+                    return false;
+            case SDLK_RIGHT:
+                if (!(SDL_GetModState() & KMOD_ALT)) {
+                     set_current (iselected+1);
+                     break;
+                } else
+                    return false;
+            case SDLK_DOWN:  set_current (iselected+width); break;
+            case SDLK_UP:    set_current (iselected>width ? iselected-width : 0); break;
+            case SDLK_PAGEDOWN: page_down(); break;
+            case SDLK_PAGEUP: page_up(); break;
+            case SDLK_HOME: start(); break;
+            case SDLK_END: end(); break;
+        
+            case SDLK_RETURN:
+                trigger_action();
+                break;
+        
+            default:
+                return false;           // key not handled
         }
         return true;
     }
