@@ -18,16 +18,18 @@
  */
 
 #include "gui/Menu.hh"
-#include "sound.hh"
+#include "SoundEffectManager.hh"
+#include "MusicManager.hh"
 #include "video.hh"
 #include "options.hh"
+#include "main.hh"
 #include "nls.hh"
-#include "ecl.hh"
+#include "ecl_video.hh"
+
 #include <cassert>
 #include <algorithm>
 #include <iostream>
 
-
 using namespace ecl;
 using namespace std;
 
@@ -35,9 +37,13 @@ using namespace std;
 
 namespace enigma { namespace gui {
     /* -------------------- Menu -------------------- */
-    
-    Menu::Menu()
-    : active_widget(0), quitp(false), abortp(false) {
+
+Menu::Menu()
+: active_widget(NULL),
+  key_focus_widget(NULL),
+  quitp(false),
+  abortp(false),
+  previous_caption(video_engine->GetCaption()) {
     }
     
     void Menu::add(Widget *w) {
@@ -59,6 +65,25 @@ namespace enigma { namespace gui {
         abortp=true;
     }
     
+    void Menu::set_key_focus(Widget *newfocus) {
+        Widget *oldfocus = key_focus_widget;
+        key_focus_widget = newfocus;
+        if (oldfocus)
+            oldfocus->invalidate();
+    }
+    
+    bool Menu::is_key_focus(Widget *focus) {
+        return (key_focus_widget == focus);
+    }
+
+    void Menu::set_caption(const std::string &text) {
+        video_engine->SetCaption(text);
+    }
+
+    std::string Menu::get_caption() const {
+        return video_engine->GetCaption();
+    }
+
     bool Menu::manage() {
         quitp=abortp=false;
         SDL_Event e;
@@ -69,10 +94,13 @@ namespace enigma { namespace gui {
             SCREEN->flush_updates();
             while (SDL_PollEvent(&e)) {
                 handle_event(e);
+                if (app.bossKeyPressed) return true;
             }
             SDL_Delay(10);
             if(active_widget) active_widget->tick(0.01);
-            tick (0.01);
+            if(key_focus_widget && (key_focus_widget != active_widget)) key_focus_widget->tick(0.01);
+            tick(0.01);
+            sound::MusicTick(0.01);
             refresh();
         }
         sound::EmitSoundEvent ("menuexit");
@@ -110,53 +138,89 @@ namespace enigma { namespace gui {
         }
     }
     
-    void Menu::handle_event(const SDL_Event &e) 
-    {
-        if (e.type == SDL_KEYDOWN && 
-            e.key.keysym.sym == SDLK_RETURN && 
-            e.key.keysym.mod & KMOD_ALT)
-        {
-            video::ToggleFullscreen();
+    void Menu::handle_event(const SDL_Event &e) {
+        
+        // Alt && Return for Fullscreen Toggle on Linux only
+        if (e.type == SDL_KEYDOWN &&  e.key.keysym.sym == SDLK_RETURN && 
+                e.key.keysym.mod & KMOD_ALT) {
+            video_engine->ToggleFullscreen();
             return;
         }
-    
-        if (on_event(e))
+        
+        // Boss quit key Shift && ESC or Mac OS X application quit sequence
+        if (e.type == SDL_KEYDOWN && ((e.key.keysym.sym == SDLK_ESCAPE && 
+                (e.key.keysym.mod & KMOD_SHIFT)) || (e.key.keysym.sym == SDLK_q && 
+                        (e.key.keysym.mod & (KMOD_LGUI | KMOD_RGUI))))) {
+            abort();
+            app.bossKeyPressed = true;
+            return;
+        }
+
+        // Cycle through languages in wizard mode on Shift+Page-Up/Down
+        if (   enigma::WizardMode
+            && e.type == SDL_KEYDOWN
+            && (e.key.keysym.sym == SDLK_PAGEUP || e.key.keysym.sym == SDLK_PAGEDOWN)
+            && (e.key.keysym.mod & KMOD_SHIFT)) {
+            nls::CycleLocale(e.key.keysym.sym == SDLK_PAGEUP);
+            invalidate_all();
+        }
+
+        // first allow active widget to handle event
+        if (active_widget && active_widget->on_event(e))
+            return;
+        
+        // key focus handler next to catch key events
+        if (key_focus_widget && key_focus_widget->on_event(e))
+            return;
+        
+        // menu subclass with special handling
+        if (on_event(e) && e.type != SDL_MOUSEMOTION)   // track active widgets (LevelWidget)
             return;
     
         switch (e.type) {
-        case SDL_QUIT:
-            abort();
-            break;
-        case SDL_MOUSEMOTION:
-            track_active_widget( e.motion.x, e.motion.y );
-            break;
-        case SDL_KEYDOWN:
-            if (!active_widget || !active_widget->on_event(e)) {
-                // if not handled by active_widget
-                switch (e.key.keysym.sym) {
-                case SDLK_ESCAPE:
-                    abort();
+            case SDL_QUIT:
+                abort();
+                app.bossKeyPressed = true;
+                break;
+            case SDL_MOUSEMOTION:
+                track_active_widget( e.motion.x, e.motion.y );
+                break;
+            case SDL_KEYDOWN:
+                if (!active_widget || !active_widget->on_event(e)) {
+                    // if not handled by active_widget
+                    switch (e.key.keysym.sym) {
+                    case SDLK_ESCAPE:
+                        abort();
+                        break;
+                    // TODO replace cursor keys with tab for gotos
+                    case SDLK_DOWN:  goto_adjacent_widget( 0,  1); break;
+                    case SDLK_UP:    goto_adjacent_widget( 0, -1); break;
+                    case SDLK_RIGHT: goto_adjacent_widget( 1,  0); break;
+                    case SDLK_LEFT:  goto_adjacent_widget(-1,  0); break;
+                    default:
+                        break;
+                    }
+                }
+                break;
+            case SDL_MOUSEBUTTONDOWN:
+            case SDL_MOUSEBUTTONUP:
+                track_active_widget( e.button.x, e.button.y );
+                if (active_widget) active_widget->on_event(e);
+                break;
+            case SDL_WINDOWEVENT:
+                switch (e.window.event) {
+                case SDL_WINDOWEVENT_EXPOSED:
+                    draw_all();
                     break;
-                case SDLK_DOWN:  goto_adjacent_widget( 0,  1); break;
-                case SDLK_UP:    goto_adjacent_widget( 0, -1); break;
-                case SDLK_RIGHT: goto_adjacent_widget( 1,  0); break;
-                case SDLK_LEFT:  goto_adjacent_widget(-1,  0); break;
-                default:
+                case SDL_WINDOWEVENT_RESIZED:
+                    video_engine->Resize(e.window.data1, e.window.data2);
+                    // Drawing will be done by tick.
+                    draw_all();
                     break;
                 }
-            }
-    
-            break;
-        case SDL_MOUSEBUTTONDOWN:
-        case SDL_MOUSEBUTTONUP:
-            track_active_widget( e.button.x, e.button.y );
-            if (active_widget) active_widget->on_event(e);
-            break;
-        case SDL_VIDEOEXPOSE:
-            draw_all();
-            break;
-        default:
-            if (active_widget) active_widget->on_event(e);
+                break;
+            default:
+                if (active_widget) active_widget->on_event(e);
         }
     }
     
@@ -170,12 +234,14 @@ namespace enigma { namespace gui {
         }
     }
     
-    void Menu::track_active_widget( int x, int y ) {
+    void Menu::track_active_widget( int window_x, int window_y ) {
+        int x = (int)((double) (window_x * SCREEN->size().w) / SCREEN->window_size().w + 0.5);
+        int y = (int)((double) (window_y * SCREEN->size().h) / SCREEN->window_size().h + 0.5);
         switch_active_widget(find_widget(x, y));
     }
-    
-    
-    void Menu::center() {
+
+    // Center all widgets on the screen, with given margins (default 0).
+    void Menu::center(int top, int bottom, int left, int right) {
         if (m_widgets.size() > 0) {
             using std::min;
             using std::max;
@@ -189,7 +255,12 @@ namespace enigma { namespace gui {
                 a.w += max(0, r.x+r.w-a.x-a.w);
                 a.h += max(0, r.y+r.h-a.y-a.h);
             }
-            Rect c=ecl::center(SCREEN->size(), a);
+            Rect b = SCREEN->size();
+            b.x += left;
+            b.y += top;
+            b.w -= left + right;
+            b.h -= top + bottom;
+            Rect c = ecl::center(b, a);
             int dx = c.x-a.x;
             int dy = c.y-a.y;
     
@@ -203,7 +274,7 @@ namespace enigma { namespace gui {
             }
         }
     }
-    
+
     void Menu::draw (ecl::GC &gc, const ecl::Rect &r)
     {
         clip(gc, r);
