@@ -20,13 +20,14 @@
 #include "main.hh"
 #include "gui/widgets.hh"
 #include "gui/TextField.hh"
+#include "ecl_font.hh"
 #include "ecl_utf.hh"
-#include "enigma.hh"
-#include "sound.hh"
+#include "ecl_video.hh"
+#include "SoundEffectManager.hh"
 #include "video.hh"
 #include "options.hh"
+#include "resource_cache.hh"
 #include "nls.hh"
-#include "ecl.hh"
 #include <cassert>
 #include <algorithm>
 #include <iostream>
@@ -37,20 +38,28 @@ using namespace std;
 
 #define SCREEN ecl::Screen::get_instance()
 
-ecl::Font *TextField::menufont = 0;
-
 TextField::TextField(const std::string &t, ActionListener *al) : cursorTime(0),
-        showCursor(true), isLastActionReturn (false), invalidChars(""),
-        isLimitedToValidChars(false), maxChars(-1) {
-    if (menufont == 0) {
-        menufont = enigma::GetFont("menufont");
-    }
+        showCursor(true), isLimitedToValidChars(false), invalidChars(""), 
+        maxChars(-1), isLastActionReturn (false) {
+    menufont = enigma::GetFont("menufont");
+    
+    SDL_StartTextInput();
     set_listener(al);
     textPreCursor = t;
     ecl::utf8CharSizes(textPreCursor, charSizesPreCursor);
     textPostCursor= "";
 }
 
+TextField::~TextField() {
+    SDL_StopTextInput();
+}
+
+void TextField::activate() {
+    Button::activate();
+    if (get_parent())
+        get_parent()->set_key_focus(this);
+}
+
 void TextField::set_text(const std::string &t) {
     textPreCursor = t;
     charSizesPreCursor.clear();
@@ -72,6 +81,7 @@ void TextField::setInvalidChars(std::string forbiddenChars) {
 void TextField::setMaxChars(int max) {
     maxChars = max;
 }
+
 bool TextField::wasLastActionReturn() {
     return isLastActionReturn;
 }
@@ -104,10 +114,10 @@ void TextField::draw(ecl::GC &gc, const ecl::Rect &r) {
         // cursor would be right of textfiled - shift centered text left
         x = get_x() + get_w() - 5 - w_pre;
     
-    f->render (gc, x, y, textPreCursor.c_str());
+    f->render (gc, x, y, textPreCursor);
     
     x += w_pre; 
-    if (m_activep) {
+    if (m_activep || get_parent()->is_key_focus(this)) {
         if (showCursor) {
             set_color(gc, 200,200,200);
             vline(gc, x, y, h);
@@ -115,25 +125,61 @@ void TextField::draw(ecl::GC &gc, const ecl::Rect &r) {
         x += w_cursor;
     }
     
-    f->render (gc, x, y, textPostCursor.c_str());
+    f->render(gc, x, y, textPostCursor);
    
 }
 
 bool TextField::on_event(const SDL_Event &e) {
-    bool handeled = false;
+    bool handled = false;
     bool modified = false;
     
     switch (e.type) {
         case SDL_MOUSEBUTTONDOWN:
             // set cursor
             break;
+        case SDL_TEXTINPUT: {
+            /* Add new text onto the end of our text */
+            //strcat(text, e.text.text);
+            std::string newText(e.text.text);
+            int totalLength = textPreCursor.length() + newText.length() + textPostCursor.length();
+            if (   (maxChars >= 0 && totalLength >= maxChars)
+                || (newText.length() == 1 && invalidChars.find((char)(newText[0])) != std::string::npos)) {
+                // string too long or invalid char
+                sound::EmitSoundEvent ("menustop");
+                break;
+            }
+            textPreCursor += newText;
+            charSizesPreCursor.push_back(newText.length());
+            invalidate();
+            modified = true;
+            handled = true;
+            break;
+        }
+        /* TODO: The following needs testing by someone from the CJK area.
+                 Also: charSizesPreCursor has to be adapted.
+        case SDL_TEXTEDITING: {
+            //Update the composition text.
+            //Update the cursor position.
+            //Update the selection length (if any).
+            const char *composition;
+            Sint32 cursor;
+            Sint32 selection_len;
+            composition = e.edit.text;
+            cursor = e.edit.start;
+            selection_len = e.edit.length;
+            invalidate();
+            modified = true;
+            handled = true;
+            break;
+        } */
         case SDL_KEYDOWN:
             switch (e.key.keysym.sym) {
                 case SDLK_RETURN:
-                    handeled = true;
+                case SDLK_KP_ENTER:
+                    handled = true;
                     isLastActionReturn = true;
                     invoke_listener();
-                    break;                    
+                    break;
                 case SDLK_RIGHT:
                     if(textPostCursor.size() > 0) {
                         int size = charSizesPostCursor.back();
@@ -143,7 +189,7 @@ bool TextField::on_event(const SDL_Event &e) {
                         textPostCursor.erase(0, size); 
                     }
                     invalidate();
-                    handeled = true;
+                    handled = true;
                     break;
                 case SDLK_LEFT:
                     if(textPreCursor.size() > 0) {
@@ -154,10 +200,10 @@ bool TextField::on_event(const SDL_Event &e) {
                         textPreCursor.erase(textPreCursor.size() - size); 
                     }
                     invalidate();
-                    handeled = true;
+                    handled = true;
                     break;
                 case SDLK_INSERT:
-                    handeled = true;
+                    handled = true;
                     break;
                 case SDLK_HOME:
                     if(textPreCursor.size() > 0) {
@@ -172,7 +218,7 @@ bool TextField::on_event(const SDL_Event &e) {
                         textPreCursor.clear(); 
                     }
                     invalidate();
-                    handeled = true;
+                    handled = true;
                     break;
                 case SDLK_END:
                     if(textPostCursor.size() > 0) {
@@ -188,7 +234,7 @@ bool TextField::on_event(const SDL_Event &e) {
                         textPostCursor.clear(); 
                     }
                     invalidate();
-                    handeled = true;
+                    handled = true;
                     break;
                 case SDLK_DELETE:
                     if(textPostCursor.size() > 0) {
@@ -197,17 +243,17 @@ bool TextField::on_event(const SDL_Event &e) {
                         charSizesPostCursor.pop_back();
                     }
                     invalidate();
-                    handeled = true;
+                    handled = true;
                     modified = true;
                     break;
                 case SDLK_BACKSPACE:
                     if(textPreCursor.size() > 0) {
-                       int size = charSizesPreCursor.back();
-                        textPreCursor.erase(textPreCursor.size() - size); 
+                        int size = charSizesPreCursor.back();
+                        textPreCursor.erase(textPreCursor.size() - size, size); 
                         charSizesPreCursor.pop_back();
                     }
                     invalidate();
-                    handeled = true;
+                    handled = true;
                     modified = true;
                     break;
                 case SDLK_ESCAPE:
@@ -215,58 +261,6 @@ bool TextField::on_event(const SDL_Event &e) {
                 case SDLK_UP:
                     // menu active widget movements
                     break;
-                default:
-                    // get char
-                    if (e.key.keysym.unicode != 0 ) {
-                        UTF16 realChar;
-                        if (e.key.keysym.unicode >= 0x20 && 
-                                (e.key.keysym.unicode < 0x80 ||   // key pad
-                                e.key.keysym.sym < 0x80)) {       // windows umlaute
-                            // the unicode is correct in these cases
-                            realChar = e.key.keysym.unicode;
-                        }
-                        else if (e.key.keysym.unicode >= 0x80 &&
-                                e.key.keysym.sym < 0x100) {
-                            // Linux: bad unicode but sym is ISO-8859-1
-                            
-                            // incomplete workaround - runs only for some lower
-                            // case umlauts
-                            // we would need to handle shift key in language
-                            // dependent manner -- or fix SDL Linux
-                            realChar = e.key.keysym.sym;
-                        }
-                        else {
-                            // chars like ctrl-a - ctrl-z
-                            sound::EmitSoundEvent ("menustop");
-                            break;
-                        }
-                        if ((maxChars >= 0 && (charSizesPreCursor.size() + charSizesPostCursor.size()) >= maxChars) ||
-                                realChar < 0x100 && invalidChars.find((char)realChar) != std::string::npos) {
-                            // string too long or invalid char
-                            sound::EmitSoundEvent ("menustop");
-                            break;
-                        }
-                        unsigned char utf8Char[4];
-                        UTF16 const * utf16Ptr = (UTF16 *)&realChar;
-                        UTF8 * utf8Ptr = utf8Char;
-                        ConversionResult result;
-                        result = ConvertUTF16toUTF8 (&utf16Ptr, utf16Ptr + 1,
-                                &utf8Ptr, utf8Char + 4, strictConversion);
-                        *utf8Ptr = 0;
-                        textPreCursor += (const char *)utf8Char;
-                        charSizesPreCursor.push_back(utf8Ptr - utf8Char);
-
-                        invalidate();
-                        handeled = true;
-                        modified = true;
-                        break;
-                    }
-                    if (e.key.keysym.sym < 300 || e.key.keysym.sym > 314 ){
-                        // chars like PageUp, F1 but not key state modifier
-                        // like shift, alt,...
-                        sound::EmitSoundEvent ("menustop");
-                    }
-                    break;
             }
             break;
         default:
@@ -276,6 +270,6 @@ bool TextField::on_event(const SDL_Event &e) {
         isLastActionReturn = false;
         invoke_listener();
     }
-    return handeled;
+    return handled;
 }
 
