@@ -22,9 +22,10 @@
 #include "SoundEffectManager.hh"
 #include "video.hh"
 #include "options.hh"
+#include "resource_cache.hh"
 #include "nls.hh"
-#include "ecl.hh"
 #include "ecl_font.hh"
+#include "ecl_video.hh"
 #include <cassert>
 #include <algorithm>
 #include <iostream>
@@ -87,8 +88,6 @@ bool Widget::on_event(const SDL_Event &e) {
 /* -------------------- Image -------------------- */
 
 void Image::draw (ecl::GC &gc, const ecl::Rect &/*r*/) {
-//    if (ecl::Surface *s = enigma::GetImage(imgname.c_str()))
-//        blit(gc, get_x(), get_y(), s);
     if (ecl::Surface *s = enigma::GetImage(imgname.c_str())) {
         int w=s->width();
         int h=s->height();
@@ -118,13 +117,13 @@ void AreaManager::invalidate_all() {
 
 void AreaManager::refresh() {
     if (!dirtyrects.empty()) {
-        video::HideMouse();
+        video_engine->HideMouse();
         GC gc(SCREEN->get_surface());
         for (RectList::iterator i = dirtyrects.begin(); i!=dirtyrects.end(); ++i) {
             top_container->draw(gc, *i);
             SCREEN->update_rect(*i);
         }
-        video::ShowMouse();
+        video_engine->ShowMouse();
         dirtyrects.clear();
     }
 }
@@ -576,7 +575,7 @@ void Label::draw (ecl::GC &gc, const ecl::Rect &)
     case VALIGN_CENTER: y += (get_h()-h)/2; break;
     }
     // translate if not an empty string
-    f->render (gc, x, y, m_text == "" ? "" : get_text().c_str());
+    f->render (gc, x, y, (m_text == "") ? std::string("") : get_text());
 }
 
 void Label::set_alignment (HAlignment halign, VAlignment valign) {
@@ -709,7 +708,7 @@ PushButton::PushButton() : m_pressedp (false) {
 bool PushButton::on_event(const SDL_Event &e) {
     Widget::on_event(e);
     bool was_pressed = m_pressedp;
-    bool handeled = false;
+    bool handled = false;
 
     switch (e.type) {
         case SDL_KEYDOWN:
@@ -717,25 +716,38 @@ bool PushButton::on_event(const SDL_Event &e) {
             // fall-through
         case SDL_MOUSEBUTTONDOWN:
             m_pressedp = true;
-            handeled = true;
+            handled = true;
             break;
-    
         case SDL_KEYUP:
             if (e.key.keysym.sym != SDLK_RETURN) break;
             lastUpSym = e.key.keysym.sym;
-            lastUpBotton = 0;
+            lastUpButton = 0;
             m_pressedp = false;
-            handeled = true;
+            handled = true;
             break;
         case SDL_MOUSEBUTTONUP:
             lastUpSym = SDLK_UNKNOWN;
-            lastUpBotton = e.button.button;
+            lastUpButton = e.button.button;
             m_pressedp = false;
-            handeled = true;
+            handled = true;
+            break;
+        case SDL_MOUSEWHEEL:
+            // Mousewheel is interpreted as left and right
+            // mouse button clicks.
+            lastUpSym = SDLK_UNKNOWN;
+            lastUpButton = 0;
+            handled = true;
+            if (e.wheel.y > 0)
+                lastUpButton = SDL_BUTTON_RIGHT;
+            if (e.wheel.y < 0)
+                lastUpButton = SDL_BUTTON_LEFT;
+            if (e.wheel.y == 0)
+                handled = false;
             break;
     }
 
-    bool changed = (was_pressed != m_pressedp);
+    bool changed =    (was_pressed != m_pressedp)
+                   || (e.type == SDL_MOUSEWHEEL);
     if (changed) {
         invalidate();
         if (!m_pressedp) {
@@ -745,23 +757,23 @@ bool PushButton::on_event(const SDL_Event &e) {
         }
     }
 
-    return handeled;
+    return handled;
 }
 
 void PushButton::deactivate() {
     m_pressedp = false;
     lastUpSym = SDLK_UNKNOWN;
-    lastUpBotton = 0;
+    lastUpButton = 0;
     invalidate();
     Button::deactivate();
 }
 
-SDLKey PushButton::getLastUpSym() {
+SDL_Keycode PushButton::getLastUpSym() {
     return lastUpSym;
 }
 
 Uint8 PushButton::getLastUpButton() {
-    return lastUpBotton;
+    return lastUpButton;
 }
 
 bool PushButton::soundOk() {
@@ -785,7 +797,7 @@ void TextButton::draw(ecl::GC &gc, const ecl::Rect &r) {
     int     x    = get_x() + (get_w()-w)/2;
     int     y    = get_y() + (get_h()-h)/2;
 
-    f->render (gc, x, y, text.c_str());
+    f->render (gc, x, y, text);
 }
 
 
@@ -835,8 +847,8 @@ BoolOptionButton::BoolOptionButton(const char    *option_name,
 }
 
 bool BoolOptionButton::toggle() {
-    bool newval = !enigma_options::GetBool(optionName);
-    enigma_options::SetOption(optionName, newval);
+    bool newval = !options::GetBool(optionName);
+    options::SetOption(optionName, newval);
     invalidate();
     return newval;
 }
@@ -846,15 +858,16 @@ void BoolOptionButton::on_action(Widget *) {
 }
 
 string BoolOptionButton::get_text() const {
-    return enigma_options::GetBool(optionName) ? _(trueText.c_str()) : _(falseText.c_str());
+    return options::GetBool(optionName) ? _(trueText.c_str()) : _(falseText.c_str());
 }
 
 /* -------------------- ValueButton -------------------- */
 
-ValueButton::ValueButton(int min_value_, int max_value_)
+ValueButton::ValueButton(int min_value_, int max_value_, ActionListener *al)
 : TextButton(this),
   min_value(min_value_),
-  max_value(max_value_)
+  max_value(max_value_),
+  secondaryListener(al)
 {
 }
 
@@ -892,22 +905,14 @@ bool ValueButton::update_value(int old_value, int new_value) {
     return false;
 }
 
-
 void ValueButton::on_action(Widget *) {
-    int incr = 1;
-    bool stop = false;
-    if (getLastUpSym() == SDLK_PAGEDOWN || 
-        getLastUpButton() == SDL_BUTTON_RIGHT ||
-        getLastUpButton() == 5) {  // wheel down
-        incr = -1;
-    }
-    if (getLastUpSym() == SDLK_PAGEDOWN || 
-        getLastUpSym() == SDLK_PAGEUP ||
-        getLastUpButton() == SDL_BUTTON_RIGHT ||
-        getLastUpButton() == SDL_BUTTON_LEFT  ||
-        getLastUpButton() ==  4 || getLastUpButton() == 5) {
-        stop = true;
-    }
+    // Mouse wheel has already been interpreted as a button click.
+    int incr = (getLastUpSym() == SDLK_PAGEDOWN ||
+                getLastUpButton() == SDL_BUTTON_RIGHT) ? -1 : 1;
+    bool stop = getLastUpSym() == SDLK_PAGEDOWN ||
+                getLastUpSym() == SDLK_PAGEUP ||
+                getLastUpButton() == SDL_BUTTON_RIGHT ||
+                getLastUpButton() == SDL_BUTTON_LEFT;
     if (inc_value(incr)) {
         sound::EmitSoundEvent("menuswitch");
     } else {
@@ -921,6 +926,8 @@ void ValueButton::on_action(Widget *) {
                 update_value(get_value(), max_value);
         }
     }
+    if (secondaryListener != NULL)
+        secondaryListener->on_action(this);
 }
 
 bool ValueButton::soundOk() {
