@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006 Ronald Lamprecht
+ * Copyright (C) 2006,2007,2008,2009 Ronald Lamprecht
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -21,13 +21,17 @@
 #include "errors.hh"
 #include "main.hh"
 #include "options.hh"
-#include "sound.hh"
+#include "SoundEffectManager.hh"
 #include "PreferenceManager.hh"
 #include "StateManager.hh"
 #include "lev/ScoreManager.hh"
 #include "lev/RatingManager.hh"
 
 #include <cstdio>
+// We do not use the random algorithm from enigma.cc, because we are
+// only concerned with shuffling a level index here -- this should not
+// interfere with the random algorithms used in the game.
+#include <random>
 
 namespace enigma { namespace lev {
 
@@ -47,6 +51,12 @@ namespace enigma { namespace lev {
         }
         currentGroup = app.state->getString("CurrentGroup");
     }
+    void Index::shutdown()
+    {
+        for (auto it = indexGroups.begin();it != indexGroups.end();it++) {
+            delete it->second;
+        }
+    }
     
     void Index::registerIndex(Index *anIndex) {
         if (anIndex == NULL)
@@ -80,7 +90,7 @@ namespace enigma { namespace lev {
         else
             groupName = anIndex->indexGroup;  // use index default group
             
-        std::vector<Index *> * group;
+        std::vector<Index *> * group = NULL;
         
         // if no prefs ask for index default group
         
@@ -94,7 +104,13 @@ namespace enigma { namespace lev {
                 group = new std::vector<Index *>;
                 indexGroups.insert(std::make_pair(groupName, group));
                 app.state->addGroup(groupName, anIndex->getName(), 0);
-                
+
+                // If this is the "Tutorials" group, move it to the front.
+                // (This is needed on transitioning from 1.21 to 1.30.)
+                if (groupName == INDEX_TUTORIALS_GROUP) {
+                    moveGroup(groupName, 0);
+                }
+
                 // fill group with indices that appear in every group
                 std::map<std::string, Index *>::iterator iti;
                 for (iti = indices.begin(); iti != indices.end(); iti++)
@@ -619,7 +635,7 @@ namespace enigma { namespace lev {
     
     bool Index::hasNormLevelPath(std::string path) {
         for (int i = 0; i < proxies.size(); i++) {
-            if (proxies[i]->getNormLevelPath() == path)
+            if (proxies[i]->getNormFilePath() == path)
                 return true;
         }
         return false;
@@ -636,6 +652,7 @@ namespace enigma { namespace lev {
             nextMode = NEXT_LEVEL_UNSOLVED;
             break;
         default:
+            // raise error
             break;
         };
 
@@ -644,7 +661,7 @@ namespace enigma { namespace lev {
         int newPos = currentPosition;
         lev::ScoreManager *scm = lev::ScoreManager::instance();
         lev::RatingManager *ratingMgr = lev::RatingManager::instance();
-        int    difficulty     = app.state->getInt("Difficulty");
+        int difficulty = app.state->getInt("Difficulty");
         
         while (newPos < max - 1 && !found) {
             ++newPos;
@@ -671,8 +688,6 @@ namespace enigma { namespace lev {
             else
                 found = true;
         }
-        if (!found)
-            newPos = 0; // ?
             
         currentPosition = newPos;
         return found;        
@@ -691,8 +706,30 @@ namespace enigma { namespace lev {
     void Index::clear() {
 //        proxies.clear();
     }
-    
+
+    void Index::sort(SCSortMethod s) {
+        switch (s) {
+            case SC_SORT_NONE: break;
+            case SC_SORT_RANDOM: {
+                std::random_device rd;
+                auto rng = std::default_random_engine { rd() };
+                std::shuffle(proxies.begin(), proxies.end(), rng);
+                break;
+            }
+            case SC_SORT_DIF: {
+                std::sort(proxies.begin(), proxies.end(), lev::RatingManager::compareByDifficulty);
+                break;
+            }
+            case SC_SORT_AVR: {
+                std::sort(proxies.begin(), proxies.end(), lev::RatingManager::compareByAverageRating);
+                break;
+            }
+            default: break;
+        }
+    }
+
     void Index::updateFromProxies() {
+        Proxy::releaseCache();   // enforce a reload from file
         for (int i = 0, l = proxies.size();  i < l; i++) {
             try {
                 proxies[i]->loadMetadata(true);
