@@ -117,7 +117,7 @@ namespace enigma { namespace lev {
             // is no reason to store the cache.
             loadURI(userRatingPath);
         }
-        if (app.prefs->getBool("RatingsAutoUpdate") == true)
+        if (!Robinson && app.prefs->getBool("RatingsAutoUpdate") == true)
             update();
     }
     
@@ -132,7 +132,8 @@ namespace enigma { namespace lev {
             app.domParserSchemaResolver->resetResolver();
             app.domParserSchemaResolver->addSchemaId("ratings.xsd","ratings.xsd");
             DOMDocument *doc = app.domParser->parseURI(path.c_str());
-            if (doc != NULL && !app.domParserErrorHandler->getSawErrors()) {
+            if (app.domParserSchemaResolver->didResolveSchema() && doc != NULL 
+                    && !app.domParserErrorHandler->getSawErrors()) {
                 DOMElement *updateElem = 
                         dynamic_cast<DOMElement *>(doc->getElementsByTagName(
                         Utf8ToXML("update").x_str())->item(0));
@@ -510,16 +511,16 @@ namespace enigma { namespace lev {
     }
 
     short RatingManager::getDifficulty(Proxy *levelProxy) {
-        int difficulty = 7*getIntelligence(levelProxy) + 
-                6*getDexterity(levelProxy) +
-                4*getPatience(levelProxy) +
-                3*getKnowledge(levelProxy) +
-                4*getSpeed(levelProxy) - 23;
-        return difficulty > 0 ? difficulty : 0;
+        Rating * theRating = findRating(levelProxy);
+        if (theRating != NULL) {
+            return theRating->difficulty();
+        }
+
+        return 0;
     }
     
     short RatingManager::getBestScore(Proxy *levelProxy, int difficulty) {
-        if (difficulty == DIFFICULTY_EASY && !levelProxy->hasEasymode())
+        if (difficulty == DIFFICULTY_EASY && !levelProxy->hasEasyMode())
             difficulty = DIFFICULTY_HARD;
         switch (difficulty) {
             case DIFFICULTY_EASY:
@@ -556,7 +557,7 @@ namespace enigma { namespace lev {
     }
     
     std::string RatingManager::getBestScoreHolder(Proxy *levelProxy, int difficulty, int cut) {
-        if (difficulty == DIFFICULTY_EASY && !levelProxy->hasEasymode())
+        if (difficulty == DIFFICULTY_EASY && !levelProxy->hasEasyMode())
             difficulty = DIFFICULTY_HARD;
         switch (difficulty) {
             case DIFFICULTY_EASY:
@@ -603,7 +604,7 @@ namespace enigma { namespace lev {
     
     
     short RatingManager::getParScore(Proxy *levelProxy, int difficulty) {
-        if (difficulty == DIFFICULTY_EASY && !levelProxy->hasEasymode())
+        if (difficulty == DIFFICULTY_EASY && !levelProxy->hasEasyMode())
             difficulty = DIFFICULTY_HARD;
         switch (difficulty) {
             case DIFFICULTY_EASY:
@@ -695,4 +696,25 @@ namespace enigma { namespace lev {
         }
         return s;
     }
+
+    bool RatingManager::compareByDifficulty(Proxy *proxy1, Proxy *proxy2) {
+        Rating *r1 = instance()->findRating(proxy1);
+        Rating *r2 = instance()->findRating(proxy2);
+        if (r1 == NULL)
+            return false;
+        if (r2 == NULL)
+            return true;
+        return r1->difficulty() < r2->difficulty();
+    }
+
+    bool RatingManager::compareByAverageRating(Proxy *proxy1, Proxy *proxy2) {
+        Rating *r1 = instance()->findRating(proxy1);
+        Rating *r2 = instance()->findRating(proxy2);
+        if (r1 == NULL)
+            return false;
+        if (r2 == NULL)
+            return true;
+        return r1->averageRating < r2->averageRating;
+    }
+
 }} // namespace enigma::lev
