@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006 Ronald Lamprecht
+ * Copyright (C) 2006,2007,2008,2009,2010 Ronald Lamprecht
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -138,7 +138,7 @@ namespace enigma { namespace lev {
                 hasValidStateUserId = true;
                 Log << "User id '" << stateUserId << "'\n";
             } else 
-                Log << "Bad user id '" << ecl::strf("%.4lX %.4lX %.4lX %.4lX",i1, i2,i3, i4) << "'\n";
+                Log << "Bad user id '" << ecl::strf("%.4X %.4X %.4X %.4X", i1, i2, i3, i4) << "'\n";
                 
         }
         
@@ -169,19 +169,20 @@ namespace enigma { namespace lev {
                 readFromZipStream(zipStream, content);
                 std::string score = content.str();
  #if _XERCES_VERSION >= 30000
-                std::auto_ptr<DOMLSInput> domInputScoreSource ( new Wrapper4InputSource(
-                        new MemBufInputSource(reinterpret_cast<const XMLByte *>(score.c_str()),
-                        score.size(), "", false)));
+                std::unique_ptr<DOMLSInput> domInputScoreSource(new Wrapper4InputSource(
+                    new MemBufInputSource(reinterpret_cast<const XMLByte *>(score.c_str()),
+                                          score.size(), "", false)));
                 doc = app.domParser->parse(domInputScoreSource.get());
-#else    
-                std::auto_ptr<Wrapper4InputSource> domInputScoreSource ( new Wrapper4InputSource(
-                        new MemBufInputSource(reinterpret_cast<const XMLByte *>(score.c_str()),
-                        score.size(), "", false)));
+#else
+                std::unique_ptr<Wrapper4InputSource> domInputScoreSource(new Wrapper4InputSource(
+                    new MemBufInputSource(reinterpret_cast<const XMLByte *>(score.c_str()),
+                                          score.size(), "", false)));
                 doc = app.domParser->parse(*domInputScoreSource);
 #endif
                 
             }
-            if (doc != NULL && !app.domParserErrorHandler->getSawErrors()) {
+            if (app.domParserSchemaResolver->didResolveSchema() &&  doc != NULL 
+                    && !app.domParserErrorHandler->getSawErrors()) {
                 propertiesElem = dynamic_cast<DOMElement *>(doc->getElementsByTagName(
                         Utf8ToXML("properties").x_str())->item(0));
                 levelsElem = dynamic_cast<DOMElement *>(doc->getElementsByTagName(
@@ -239,17 +240,7 @@ namespace enigma { namespace lev {
                         hasValidUserId = true;
                     } else {
                         // create first part of user id based on time stamp
-                        unsigned id;
-                        if (RAND_MAX > 0x10000) {
-                            id = std::rand() & 0xFFFFFFFF;
-                        } else {
-                            // MinGW 3.4.2 and maybe other configs
-                            unsigned t = std::time(NULL);
-                            std::srand(t >> 15);
-                            id = (std::rand() << 16) ^ (t & 0xFFFF);
-                            enigma::Randomize();
-                        }
-                        userId = ecl::strf("%.8lX",id);
+                        userId = ecl::strf("%.8X", Rand(false));
                         // we need a second random part as 2 users may start Enigma
                         // within the same second - we postpone this part till we save
                     }
@@ -298,8 +289,10 @@ namespace enigma { namespace lev {
                 }
                 
             }
-            if(app.domParserErrorHandler->getSawErrors()) {
+            if (app.domParserErrorHandler->getSawErrors()) {
                 errMessage = errStream.str();
+            } else if (!app.domParserSchemaResolver->didResolveSchema()) {
+                errMessage = "Wrong XML document - expected a score, got something else!";
             }
             app.domParserErrorHandler->reportToNull();  // do not report to errStream any more
         }
@@ -332,7 +325,7 @@ namespace enigma { namespace lev {
         s2 >> std::hex >> i2;
         i3 = id3 & 0xFFFF;
         i4 = (i1 ^ i2 ^ i3);
-        userId += ecl::strf("%.4lX%.4lX",i3, i4);
+        userId += ecl::strf("%.4X%.4X",i3, i4);
         app.state->setProperty("UserId", userId);
         setProperty("UserId", userId);
         hasValidUserId = true;
@@ -345,7 +338,7 @@ namespace enigma { namespace lev {
         
         while (len--)
             r = (r<<8 & 0xFFFF) ^ ctab[(r >> 8) ^ *p++];
-        return ecl::strf("%.4lX", r); 
+        return ecl::strf("%.4X", r); 
     }
 
     bool ScoreManager::save() {
@@ -368,15 +361,15 @@ namespace enigma { namespace lev {
             Log << "ReId Windows 1.00 User Id: " << userId << "\n";
             setProperty("UserId1.00", userId);
             app.state->setProperty("UserId1.00", userId);
-            unsigned id1 = std::rand() & 0xFFFF;
-            userId.replace(0, 4, ecl::strf("%.4lX",id1));
+            unsigned id1 = Rand(false) & 0xFFFF;
+            userId.replace(0, 4, ecl::strf("%.4X",id1));
             unsigned id2, id3, id4;
             std::istringstream s2(userId.substr(4, 4));
             std::istringstream s3(userId.substr(8, 4));
             s2 >> std::hex >> id2;
             s3 >> std::hex >> id3;
             id4 = (id1 ^ id2 ^ id3);
-            userId.replace(12, 4, ecl::strf("%.4lX",id4));
+            userId.replace(12, 4, ecl::strf("%.4X",id4));
             app.state->setProperty("UserId", userId);
             setProperty("UserId", userId);
             Log << "new id: " << userId << "\n";
@@ -440,7 +433,9 @@ namespace enigma { namespace lev {
 //            result = app.domSer->writeToURI(doc, LocalToXML(& path).x_str());
                 if (j==1)
                     (app.domSer)->setFilter(&serialFilter);
-                std::string contents(XMLtoUtf8(app.domSer->writeToString(doc)).c_str());
+                XMLCh* XMLString = app.domSer->writeToString(doc);
+                std::string contents(XMLtoUtf8(XMLString).c_str());
+                XMLString::release(&XMLString); 
                 if (j==1)
                     (app.domSer)->setFilter(NULL);
                 contents.replace(contents.find("UTF-16"), 6, "UTF-8"); // adapt encoding info
@@ -534,7 +529,7 @@ namespace enigma { namespace lev {
     bool ScoreManager::isSolved(lev::Proxy *levelProxy, int difficulty) {
         ecl::Assert <XFrontend> (difficulty >= DIFFICULTY_EASY &&  
                 difficulty <= DIFFICULTY_ANY, "ScoreManager::isSolved illegal difficulty");
-        if (difficulty == DIFFICULTY_EASY && !levelProxy->hasEasymode())
+        if (difficulty == DIFFICULTY_EASY && !levelProxy->hasEasyMode())
             difficulty = DIFFICULTY_HARD;
         
         DOMElement * level = getLevel(levelProxy);
@@ -546,10 +541,10 @@ namespace enigma { namespace lev {
             return false;
     }
 
-    bool ScoreManager::isOutdated(lev::Proxy *levelProxy, int difficulty) {
+    bool ScoreManager::isObsolete(lev::Proxy *levelProxy, int difficulty) {
         ecl::Assert <XFrontend> (difficulty >= DIFFICULTY_EASY &&  
                 difficulty <= DIFFICULTY_HARD, "ScoreManager::isOutdated illegal difficulty");
-        if (difficulty == DIFFICULTY_EASY && !levelProxy->hasEasymode())
+        if (difficulty == DIFFICULTY_EASY && !levelProxy->hasEasyMode())
             difficulty = DIFFICULTY_HARD;
         
         DOMElement * level = getLevel(levelProxy);
@@ -565,10 +560,40 @@ namespace enigma { namespace lev {
             return false;
     }
 
+    bool ScoreManager::isOutdated(lev::Proxy *levelProxy, int difficulty) {
+        DOMElement * level = getCreateLevel(levelProxy);
+        if (level == NULL || XMLString::parseInt(level->getAttribute(
+                Utf8ToXML("version").x_str())) != levelProxy->getScoreVersion()) {
+            return false;  // unsolved, thus not outdated
+        }
+        const XMLCh *attr = level->getAttribute(Utf8ToXML((difficulty == DIFFICULTY_HARD) ? "diff1" : "easy1").x_str());
+        int score1 = (XMLString::stringLen(attr) > 0) ? XMLString::parseInt(attr) : -1;
+        if (score1 < 0) return false;
+        attr = level->getAttribute(Utf8ToXML((difficulty == DIFFICULTY_HARD) ? "diff1rel" : "easy1rel").x_str());
+        double rel1 = 0.92; // default
+        if (XMLString::stringLen(attr) > 0) {
+            XMLDouble * result = new XMLDouble(attr);
+            rel1 = result->getValue();
+            delete result;
+        }
+        if (rel1 >= ASSURED_RELEASE) return false;
+
+        attr = level->getAttribute(Utf8ToXML((difficulty == DIFFICULTY_HARD) ? "diff2" : "easy2").x_str());
+        int score2 = (XMLString::stringLen(attr) > 0) ? XMLString::parseInt(attr) : -1;
+        attr = level->getAttribute(Utf8ToXML((difficulty == DIFFICULTY_HARD) ? "diff2rel" : "easy2rel").x_str());
+        double rel2 = 0.92; // default
+        if (XMLString::stringLen(attr) > 0) {
+            XMLDouble * result = new XMLDouble(attr);
+            rel2 = result->getValue();
+            delete result;
+        }
+        return (score2 >= 0 && rel2 >= ASSURED_RELEASE) ? false : true;
+    }
+
     int ScoreManager::getBestUserScore(lev::Proxy *levelProxy, int difficulty) {
         ecl::Assert <XFrontend> (difficulty >= DIFFICULTY_EASY &&  
                 difficulty <= DIFFICULTY_HARD, "ScoreManager::getBestUserScore illegal difficulty");
-        if (difficulty == DIFFICULTY_EASY && !levelProxy->hasEasymode())
+        if (difficulty == DIFFICULTY_EASY && !levelProxy->hasEasyMode())
             difficulty = DIFFICULTY_HARD;
         
         DOMElement * level = getLevel(levelProxy);
@@ -578,7 +603,28 @@ namespace enigma { namespace lev {
         }
         const XMLCh *attr = level->getAttribute(Utf8ToXML((difficulty == DIFFICULTY_HARD) ? "diff1" : "easy1").x_str());
         int score = (XMLString::stringLen(attr) > 0) ? XMLString::parseInt(attr) : -1;
-        return (score < 0) ? SCORE_UNSOLVED : score;
+        if (score < 0) return SCORE_UNSOLVED;
+
+        attr = level->getAttribute(Utf8ToXML((difficulty == DIFFICULTY_HARD) ? "diff1rel" : "easy1rel").x_str());
+        double rel1 = 0.92; // default
+        if (XMLString::stringLen(attr) > 0) {
+            XMLDouble * result = new XMLDouble(attr);
+            rel1 = result->getValue();
+            delete result;
+        }
+        if (rel1 >= TRUSTED_RELEASE) return score;
+
+        attr = level->getAttribute(Utf8ToXML((difficulty == DIFFICULTY_HARD) ? "diff2" : "easy2").x_str());
+        int score2 = (XMLString::stringLen(attr) > 0) ? XMLString::parseInt(attr) : -1;
+        if (score2 < 0) return SCORE_UNSOLVED;
+        attr = level->getAttribute(Utf8ToXML((difficulty == DIFFICULTY_HARD) ? "diff2rel" : "easy2rel").x_str());
+        double rel2 = 0.92; // default
+        if (XMLString::stringLen(attr) > 0) {
+            XMLDouble * result = new XMLDouble(attr);
+            rel2 = result->getValue();
+            delete result;
+        }
+        return (rel1 >= TRUSTED_RELEASE) ? score2 : SCORE_UNSOLVED;
     }
 
     void ScoreManager::updateUserScore(lev::Proxy *levelProxy, int difficulty, 
@@ -588,16 +634,20 @@ namespace enigma { namespace lev {
         if (levelProxy->getLevelStatus() != lev::STATUS_RELEASED)
             return;
         
-        if (difficulty == DIFFICULTY_EASY && !levelProxy->hasEasymode())
+        if (difficulty == DIFFICULTY_EASY && !levelProxy->hasEasyMode())
             difficulty = DIFFICULTY_HARD;
         
-        if (score > SCORE_MAX)
-            score = SCORE_MAX;  // distinguish from SCORE_SOLVED levels
-            
+        if (score > SCORE_MAX2)
+            score = SCORE_MAX2;  // distinguish from SCORE_SOLVED levels
+        
+        // limit API 1 levels to 99'59", avoiding display problems within older releases
+        if (levelProxy->getEnigmaCompatibility() < 1.10 && score > SCORE_MAX1)
+            score = SCORE_MAX1;
+        
         if (!hasValidUserId) {
             finishUserId(std::time(NULL) & 0xFFFF);
         }
-        
+
         DOMElement * level = getCreateLevel(levelProxy);
         
         // get the current newest scoreversion of the level - it exists now
@@ -672,13 +722,13 @@ namespace enigma { namespace lev {
             } else if (score < score2) {
                 // score better than previous second best score
                 if (rel2 <= enigmaRelease) {
+                    // limit score hunt to newest enigma release
                     score2 = score;
                     rel2 = enigmaRelease;
                     score2mod = true;
                 }
-            } else if (rel2 < enigmaRelease) {
-                // we remember the second best score played with the most recent 
-                // Enigma version
+            } else if (rel2 < TRUSTED_RELEASE) {
+                // we remember the second best score played with a trusted release
                 score2 = score;
                 rel2 = enigmaRelease;
                 score2mod = true;
@@ -702,7 +752,7 @@ namespace enigma { namespace lev {
     }
     
     bool ScoreManager::bestScoreReached(lev::Proxy *levelProxy, int difficulty) {
-        if (difficulty == DIFFICULTY_EASY && !levelProxy->hasEasymode())
+        if (difficulty == DIFFICULTY_EASY && !levelProxy->hasEasyMode())
             difficulty = DIFFICULTY_HARD;
         
         int bestUserScore = getBestUserScore(levelProxy, difficulty);
@@ -711,7 +761,7 @@ namespace enigma { namespace lev {
     }
     
     bool ScoreManager::parScoreReached(lev::Proxy *levelProxy, int difficulty) {
-        if (difficulty == DIFFICULTY_EASY && !levelProxy->hasEasymode())
+        if (difficulty == DIFFICULTY_EASY && !levelProxy->hasEasyMode())
             difficulty = DIFFICULTY_HARD;
         
         int bestUserScore = getBestUserScore(levelProxy, difficulty);
@@ -751,7 +801,7 @@ namespace enigma { namespace lev {
                     return;  // avoid deleting existing scores
             }
             // reset the score to solved but no score value
-            updateUserScore(levelProxy, difficulty, 99*60+59); // store max possible score
+            updateUserScore(levelProxy, difficulty, SCORE_MAX2); // store max possible score
             level = getLevel(levelProxy);
             // check if score is created - it may be NULL if level is not released
             if (level != NULL && XMLString::parseInt(level->getAttribute(
@@ -768,7 +818,7 @@ namespace enigma { namespace lev {
         int size = ind->size();
         int num = 0;
         for (i=0 ; i < size; i++) {
-            if (isSolved(ind->getProxy(i),difficulty))
+            if (isSolved(ind->getProxy(i), difficulty))
                 num++;
         }
         return num;
@@ -779,7 +829,7 @@ namespace enigma { namespace lev {
         int size = ind->size();
         int num = 0;
         for (i=0 ; i < size; i++) {
-            if (bestScoreReached(ind->getProxy(i),difficulty))
+            if (bestScoreReached(ind->getProxy(i), difficulty))
                 num++;
         }
         return num;
@@ -790,7 +840,7 @@ namespace enigma { namespace lev {
         int size = ind->size();
         int num = 0;
         for (i=0 ; i < size; i++) {
-            if (parScoreReached(ind->getProxy(i),difficulty))
+            if (parScoreReached(ind->getProxy(i), difficulty))
                 num++;
         }
         return num;
@@ -1046,10 +1096,10 @@ namespace enigma { namespace lev {
                             easyScore != SCORE_UNSOLVED;
                     
                     // limit scores to 99*60+59 - they need to fit in a short for XML
-                    if (diffScore > SCORE_MAX)
-                        diffScore = SCORE_MAX;
-                    if (easyScore > SCORE_MAX)
-                        easyScore = SCORE_MAX;
+                    if (diffScore > SCORE_MAX1)
+                        diffScore = SCORE_MAX1;
+                    if (easyScore > SCORE_MAX1)
+                        easyScore = SCORE_MAX1;
                     
                     std::map<std::string,  DOMElement *>::iterator itx;
                     // do we have to update a score entry for a higher version ?
